@@ -7,6 +7,11 @@
 #include "MySQL_Protocol.h"
 #include "proxy_protocol_info.h"
 
+#ifndef PROXYJSON
+#define PROXYJSON
+#include "../deps/json/json_fwd.hpp"
+#endif // PROXYJSON
+
 #ifndef uchar
 typedef unsigned char uchar;
 #endif
@@ -120,7 +125,7 @@ class MySQL_Data_Stream
 	PtrSizeArray *resultset;
 	unsigned int resultset_length;
 
-	ProxySQL_Poll *mypolls;
+	ProxySQL_Poll<MySQL_Data_Stream> *mypolls;
 	//int listener;
 	MySQL_Connection *myconn;
 	MySQL_Session *sess;  // pointer to the session using this data stream
@@ -179,8 +184,7 @@ class MySQL_Data_Stream
 	char *com_field_wild;
 
 	MySQL_Data_Stream();
-	~MySQL_Data_Stream();
-
+	virtual ~MySQL_Data_Stream();
 	int array2buffer_full();
 	void init();	// initialize the data stream
 	void init(enum MySQL_DS_type, MySQL_Session *, int); // initialize with arguments
@@ -224,7 +228,7 @@ class MySQL_Data_Stream
 		//
 		// we have a similar code in MySQL_Connection
 		// in case of ASYNC_CONNECT_SUCCESSFUL
-		if (sess != NULL && sess->session_fast_forward == true) {
+		if (sess != NULL && sess->session_fast_forward) {
 			// if frontend and backend connection use SSL we will set
 			// encrypted = true and we will start using the SSL structure
 			// directly from P_MARIADB_TLS structure.
@@ -256,7 +260,7 @@ class MySQL_Data_Stream
 		myconn->myds=NULL;
 		myconn=NULL;
 		if (encrypted == true) {
-			if (sess != NULL && sess->session_fast_forward == true) {
+			if (sess != NULL && sess->session_fast_forward) {
 				// it seems we are a connection with SSL on a fast_forward session.
 				// See attach_connection() for more details .
 				// We now disable SSL metadata from the Data Stream
@@ -275,6 +279,8 @@ class MySQL_Data_Stream
 
 	bool data_in_rbio();
 
-	void get_client_myds_info_json(json&);
+	void reset_connection();
+
+	void get_client_myds_info_json(nlohmann::json&);
 };
 #endif /* __CLASS_MYSQL_DATA_STREAM_H */
