@@ -4,6 +4,11 @@
 #include "cpp.h"
 #include <atomic>
 
+#ifndef PROXYJSON
+#define PROXYJSON
+#include "../deps/json/json_fwd.hpp"
+#endif // PROXYJSON
+
 #define PROXYSQL_LOGGER_PTHREAD_MUTEX
 
 class MySQL_Logger;
@@ -75,6 +80,7 @@ private:
 	uint64_t rows_sent;        ///< Number of rows sent.
 	uint32_t client_stmt_id;   ///< Client statement ID.
 	char * gtid;               ///< GTID.
+	MySQL_Session *session;	   ///< track creating session
 	char *errmsg;              ///< Error message, if generated by ProxySQL (not if generated by the backend)
 	unsigned int myerrno;      ///< MySQL error number
 
@@ -90,10 +96,11 @@ public:
 	 * @param _query_digest The digest of the query.
 	 * @param _client The client address.
 	 * @param _client_len The length of the client address.
+	 * @param sess_ptr Pointer to the session that generated this logging event, if it exists
 	 *
 	 * This constructor initializes the MySQL_Event object with the provided parameters.  It does not allocate memory for string members.
 	 */
-	MySQL_Event(log_event_type _et, uint32_t _thread_id, char* _username, char* _schemaname, uint64_t _start_time, uint64_t _end_time, uint64_t _query_digest, char* _client, size_t _client_len);
+	MySQL_Event(log_event_type _et, uint32_t _thread_id, char* _username, char* _schemaname, uint64_t _start_time, uint64_t _end_time, uint64_t _query_digest, char* _client, size_t _client_len, MySQL_Session *sess_ptr = nullptr);
 
 	/**
 	 * @brief Copy constructor for the MySQL_Event class.
@@ -217,6 +224,13 @@ public:
 	 */
 	void set_errmsg(const unsigned int _myerrno, const char * _errmsg);
 
+	/**
+	 * @brief for STMT_EXECUTE, return the parameters used
+	 * @param j A JSON object where it will add the parameters
+	 * @return nothing
+	 */
+	void extractStmtExecuteMetadataToJson(nlohmann::json &j);
+
 	/**
 	 * @brief Declares MySQL_Logger as a friend class, granting it access to private members of MySQL_Event.
 	 */
@@ -301,7 +315,6 @@ public:
 	unsigned long long getEventsDroppedCount() const { return eventsDroppedCount; }
 };
 
-
 /**
  * @class MySQL_Logger
  * @brief A class for logging MySQL events and audit entries.
@@ -363,8 +376,8 @@ private:
 		std::atomic<unsigned long long> totalEventsCopiedToMemory;
 		/** @brief Total number of events copied to the on-disk database. */
 		std::atomic<unsigned long long> totalEventsCopiedToDisk;
-		//std::atomic<unsigned long long> eventsAddedToBufferCount;     ///< Total number of events added to the buffer.
-		//std::atomic<unsigned long long> eventsCurrentlyInBufferCount; ///< Number of events currently in the buffer.
+		std::atomic<unsigned long long> eventsAddedToBufferCount;     ///< Total number of events added to the buffer.
+		std::atomic<unsigned long long> eventsCurrentlyInBufferCount; ///< Number of events currently in the buffer.
 	} metrics;
 
 	/**
@@ -413,7 +426,6 @@ private:
 	 * @return The next available ID.
 	 */
 	unsigned int audit_find_next_id();
-
 public:
 	/**
 	 * @brief Constructor for the MySQL_Logger class.
