@@ -3,7 +3,7 @@
  * @brief Declaration of the MySQL_Session class and associated types and enums.
  */
 
-
+#ifdef CLASS_BASE_SESSION_H
 #ifndef __CLASS_MYSQL_SESSION_H
 #define __CLASS_MYSQL_SESSION_H
 
@@ -13,9 +13,12 @@
 #include "proxysql.h"
 #include "cpp.h"
 #include "MySQL_Variables.h"
+#include "Base_Session.h"
 
-#include "../deps/json/json.hpp"
-using json = nlohmann::json;
+#ifndef PROXYJSON
+#define PROXYJSON
+#include "../deps/json/json_fwd.hpp"
+#endif // PROXYJSON
 
 extern class MySQL_Variables mysql_variables;
 
@@ -23,6 +26,7 @@ extern class MySQL_Variables mysql_variables;
  * @enum proxysql_session_type
  * @brief Defines the types of ProxySQL sessions.
  */
+/*
 enum proxysql_session_type {
 	PROXYSQL_SESSION_MYSQL,
 	PROXYSQL_SESSION_ADMIN,
@@ -33,6 +37,7 @@ enum proxysql_session_type {
 
 	PROXYSQL_SESSION_NONE
 };
+*/
 
 /**
  * @enum ps_type
@@ -44,26 +49,9 @@ enum ps_type : uint8_t {
 	ps_type_execute_stmt = 0x2
 };
 
-std::string proxysql_session_type_str(enum proxysql_session_type session_type);
 
-/**
- * @class Session_Regex
- * @brief Encapsulates regex operations for session handling.
- *
- * This class is used for matching patterns in SQL queries, specifically for
- * settings like sql_log_bin, sql_mode, and time_zone.
- * See issues #509 , #815 and #816
- */
-class Session_Regex {
-	private:
-	void *opt;
-	void *re;
-	char *s;
-	public:
-	Session_Regex(char *p);
-	~Session_Regex();
-	bool match(char *m);
-};
+
+//std::string proxysql_session_type_str(enum proxysql_session_type session_type);
 
 /**
  * @class Query_Info
@@ -117,7 +105,7 @@ class Query_Info {
  * This class is central to ProxySQL's handling of client connections. It manages the lifecycle
  * of a session, processes queries, and communicates with backend MySQL servers.
  */
-class MySQL_Session
+class MySQL_Session: public Base_Session<MySQL_Session, MySQL_Data_Stream, MySQL_Backend, MySQL_Thread>
 {
 	private:
 	//int handler_ret;
@@ -165,9 +153,9 @@ class MySQL_Session
 
 	void handler___client_DSS_QUERY_SENT___server_DSS_NOT_INITIALIZED__get_connection();	
 
-	void return_proxysql_internal(PtrSize_t *);
+	//void return_proxysql_internal(PtrSize_t *);
 	bool handler_special_queries(PtrSize_t *);
-	bool handler_special_queries_STATUS(PtrSize_t *);
+	//bool handler_special_queries_STATUS(PtrSize_t *);
 	/**
 	 * @brief Handles 'COMMIT|ROLLBACK' commands.
 	 * @details Forwarding the packet is required when there are active transactions. Since we are limited to
@@ -226,15 +214,17 @@ class MySQL_Session
 	bool handler_again___status_SETTING_MULTI_STMT(int *_rc);
 	bool handler_again___multiple_statuses(int *rc);
 
-	void init();
+	//void init();
 	void reset();
 	void add_ldap_comment_to_pkt(PtrSize_t *);
+
+#if 0
 	/**
 	 * @brief Performs the required housekeeping operations over the session and its connections before
 	 *  performing any processing on received client packets.
 	 */
 	void housekeeping_before_pkts();
-
+#endif // 0
 	int get_pkts_from_client(bool&, PtrSize_t&);
 
 	// GPFC_ functions are subfunctions of get_pkts_from_client()
@@ -242,7 +232,7 @@ class MySQL_Session
 	void GPFC_DetectedMultiPacket_SetDDS();
 	int GPFC_WaitingClientData_FastForwardSession(PtrSize_t&);
 	void GPFC_PreparedStatements(PtrSize_t&, unsigned char);
-	void GPFC_Replication_SwitchToFastForward(PtrSize_t&, unsigned char);
+	int GPFC_Replication_SwitchToFastForward(PtrSize_t&, unsigned char);
 	bool GPFC_QueryUSE(PtrSize_t&, int&);
 
 	void handler___status_WAITING_CLIENT_DATA___STATE_SLEEP___MYSQL_COM_STMT_RESET(PtrSize_t&);
@@ -277,19 +267,17 @@ class MySQL_Session
 	void handler_WCD_SS_MCQ_qpo_OK_msg(PtrSize_t *pkt);
 	void handler_WCD_SS_MCQ_qpo_error_msg(PtrSize_t *pkt);
 	void handler_WCD_SS_MCQ_qpo_LargePacket(PtrSize_t *pkt);
-//	int handler_WCD_SS_MCQ_qpo_Parse_SQL_LOG_BIN(PtrSize_t *pkt, bool *lock_hostgroup, unsigned int nTrx, string& nq);
 
 	public:
 	bool handler_again___status_SETTING_GENERIC_VARIABLE(int *_rc, const char *var_name, const char *var_value, bool no_quote=false, bool set_transaction=false);
 	bool handler_again___status_SETTING_SQL_LOG_BIN(int *);
 	std::stack<enum session_status> previous_status;
-	void * operator new(size_t);
-	void operator delete(void *);
 
 	Query_Info CurrentQuery;
 	PtrSize_t mirrorPkt;
 	PtrSize_t pkt;
 
+#if 0
 	// uint64_t
 	unsigned long long start_time;
 	unsigned long long pause_until;
@@ -299,12 +287,16 @@ class MySQL_Session
 
 	// pointers
 	MySQL_Thread *thread;
-	Query_Processor_Output *qpo;
+#endif // 0
+	MySQL_Query_Processor_Output *qpo;
 	StatCounters *command_counters;
+#if 0
 	MySQL_Backend *mybe;
 	PtrArray *mybes;
 	MySQL_Data_Stream *client_myds;
+#endif // 0
 	MySQL_Data_Stream *server_myds;
+#if 0
 	/*
 	 * @brief Store the hostgroups that hold connections that have been flagged as 'expired' by the
 	 *  maintenance thread. These values will be used to release the retained connections in the specific
@@ -359,8 +351,9 @@ class MySQL_Session
 	bool schema_locked;
 	bool transaction_persistent;
 	bool session_fast_forward;
-	bool started_sending_data_to_client; // this status variable tracks if some result set was sent to the client, or if proxysql is still buffering everything
+	//bool started_sending_data_to_client; // this status variable tracks if some result set was sent to the client, or if proxysql is still buffering everything
 	bool use_ssl;
+#endif // 0
 	/**
 	 * @brief This status variable tracks whether the session is performing an
 	 *   'Auth Switch' due to a 'COM_CHANGE_USER' packet.
@@ -378,8 +371,8 @@ class MySQL_Session
 	//uint64_t gtid_trxid;
 	int gtid_hid;
 
-	MySQL_STMTs_meta *sess_STMTs_meta;
-	StmtLongDataHandler *SLDH;
+//	MySQL_STMTs_meta *sess_STMTs_meta;
+//	StmtLongDataHandler *SLDH;
 
 	Session_Regex **match_regexes;
 
@@ -392,22 +385,22 @@ class MySQL_Session
 	MySQL_Session();
 	~MySQL_Session();
 
-	void set_unhealthy();
+	//void set_unhealthy();
 	
 	void set_status(enum session_status e);
 	int handler();
 
-	void (*handler_function) (MySQL_Session *arg, void *, PtrSize_t *pkt);
-	MySQL_Backend * find_backend(int);
-	MySQL_Backend * create_backend(int, MySQL_Data_Stream *_myds=NULL);
-	MySQL_Backend * find_or_create_backend(int, MySQL_Data_Stream *_myds=NULL);
+	void (*handler_function) (MySQL_Session* sess, void *, PtrSize_t *pkt);
+	//MySQL_Backend * find_backend(int);
+	//MySQL_Backend * create_backend(int, MySQL_Data_Stream *_myds=NULL);
+	//MySQL_Backend * find_or_create_backend(int, MySQL_Data_Stream *_myds=NULL);
 	
-	void SQLite3_to_MySQL(SQLite3_result *, char *, int , MySQL_Protocol *, bool in_transaction=false, bool deprecate_eof_active=false);
+	void SQLite3_to_MySQL(SQLite3_result *, char *, int , MySQL_Protocol *, bool in_transaction=false, bool deprecate_eof_active=false) override;
 	void MySQL_Result_to_MySQL_wire(MYSQL *mysql, MySQL_ResultSet *MyRS, unsigned int warning_count, MySQL_Data_Stream *_myds=NULL);
 	void MySQL_Stmt_Result_to_MySQL_wire(MYSQL_STMT *stmt, MySQL_Connection *myconn);
-	unsigned int NumActiveTransactions(bool check_savpoint=false);
-	bool HasOfflineBackends();
-	bool SetEventInOfflineBackends();
+	//unsigned int NumActiveTransactions(bool check_savpoint=false);
+	//bool HasOfflineBackends();
+	//bool SetEventInOfflineBackends();
 	/**
 	 * @brief Finds one active transaction in the current backend connections.
 	 * @details Since only one connection is returned, if the session holds multiple backend connections with
@@ -420,15 +413,15 @@ class MySQL_Session
 	 *  https://bugs.mysql.com/bug.php?id=107875.
 	 * @returns The hostgroup in which the connection was found, -1 in case no connection is found.
 	 */
-	int FindOneActiveTransaction(bool check_savepoint=false);
+	//int FindOneActiveTransaction(bool check_savepoint=false);
 	unsigned long long IdleTime();
 
-	void reset_all_backends();
-	void writeout();
+	//void reset_all_backends();
+	//void writeout();
 	void Memory_Stats();
-	void create_new_session_and_reset_connection(MySQL_Data_Stream *_myds);
+	void create_new_session_and_reset_connection(MySQL_Data_Stream *_myds) override;
 	bool handle_command_query_kill(PtrSize_t *);
-	void update_expired_conns(const std::vector<std::function<bool(MySQL_Connection*)>>&);
+	//void update_expired_conns(const std::vector<std::function<bool(MySQL_Connection*)>>&);
 	/**
 	 * @brief Performs the final operations after current query has finished to be executed. It updates the session
 	 *  'transaction_persistent_hostgroup', and updates the 'MySQL_Data_Stream' and 'MySQL_Connection' before
@@ -442,16 +435,16 @@ class MySQL_Session
 	 *   params.
 	 */
 	void finishQuery(MySQL_Data_Stream *myds, MySQL_Connection *myconn, bool);
-	void generate_proxysql_internal_session_json(json &);
+	void generate_proxysql_internal_session_json(nlohmann::json &) override;
 	bool known_query_for_locked_on_hostgroup(uint64_t);
 	void unable_to_parse_set_statement(bool *);
-	bool has_any_backend();
+	//bool has_any_backend();
 	void detected_broken_connection(const char *file, unsigned int line, const char *func, const char *action, MySQL_Connection *myconn, int myerr, const char *message, bool verbose=false);
 	void generate_status_one_hostgroup(int hid, std::string& s);
 	void reset_warning_hostgroup_flag_and_release_connection();
-	friend void SQLite3_Server_session_handler(MySQL_Session *sess, void *_pa, PtrSize_t *pkt);
-
 	void set_previous_status_mode3(bool allow_execute=true);
+
+	friend void SQLite3_Server_session_handler(MySQL_Session*, void *_pa, PtrSize_t *pkt);
 };
 
 #define KILL_QUERY       1
@@ -473,8 +466,6 @@ public:
 	KillArgs(char *u, char *p, char *h, unsigned int P, unsigned int _hid, unsigned long i, int kt, int _use_ssl, MySQL_Thread* _mt, char *ip);
 	~KillArgs();
 	const char* get_host_address() const;
-	void resolve_hostname();
-	void remove_dns_record();
 
 private:
 	char* ip_addr;
@@ -483,3 +474,4 @@ private:
 void * kill_query_thread(void *arg);
 
 #endif /* __CLASS_MYSQL_SESSION_ H */
+#endif // CLASS_BASE_SESSION_H
