@@ -14,11 +14,13 @@
 
 #include "prometheus_helpers.h"
 
-#include "set_parser.h"
+#include "MySQL_Set_Stmt_Parser.h"
 
+/*
 #define MIN_POLL_LEN 8
 #define MIN_POLL_DELETE_RATIO  8
 #define MY_EPOLL_THREAD_MAXEVENTS 128
+*/
 
 #define ADMIN_HOSTGROUP	-2
 #define STATS_HOSTGROUP	-3
@@ -41,17 +43,6 @@ typedef struct __attribute__((aligned(64))) _conn_exchange_t {
 } conn_exchange_t;
 #endif // IDLE_THREADS
 
-typedef struct _thr_id_username_t {
-	uint32_t id;
-	char *username;
-} thr_id_usr;
-
-typedef struct _kill_queue_t {
-	pthread_mutex_t m;
-	std::vector<thr_id_usr *> conn_ids;
-	std::vector<thr_id_usr *> query_ids;
-} kill_queue_t;
-
 enum MySQL_Thread_status_variable {
 	st_var_backend_stmt_prepare,
 	st_var_backend_stmt_execute,
@@ -93,20 +84,21 @@ enum MySQL_Thread_status_variable {
 	st_var_hostgroup_locked_queries,
 	st_var_aws_aurora_replicas_skipped_during_query,
 	st_var_automatic_detected_sqli,
-	st_var_whitelisted_sqli_fingerprint,
+	st_var_mysql_whitelisted_sqli_fingerprint,
 	st_var_client_host_error_killed_connections,
-	st_var_END
+	MY_st_var_END
 };
 
-class __attribute__((aligned(64))) MySQL_Thread
+class __attribute__((aligned(64))) MySQL_Thread : public Base_Thread
 {
+	friend class PgSQL_Thread;
 	private:
 	unsigned int servers_table_version_previous;
 	unsigned int servers_table_version_current;
   unsigned long long last_processing_idles;
 	MySQL_Connection **my_idle_conns;
   bool processing_idles;
-	bool maintenance_loop;
+	//bool maintenance_loop;
 	bool retrieve_gtids_required; // if any of the servers has gtid_port enabled, this needs to be turned on too
 
 	PtrArray *cached_connections;
@@ -118,7 +110,7 @@ class __attribute__((aligned(64))) MySQL_Thread
 	std::map<unsigned int, unsigned int> sessmap;
 #endif // IDLE_THREADS
 
-	Session_Regex **match_regexes;
+	//Session_Regex **match_regexes;
 
 #ifdef IDLE_THREADS
 	void worker_thread_assigns_sessions_to_idle_thread(MySQL_Thread *thr);
@@ -128,26 +120,26 @@ class __attribute__((aligned(64))) MySQL_Thread
 	void idle_thread_check_if_worker_thread_has_unprocess_resumed_sessions_and_signal_it(MySQL_Thread *thr);
 	void idle_thread_prepares_session_to_send_to_worker_thread(int i);
 	void idle_thread_to_kill_idle_sessions();
-	bool move_session_to_idle_mysql_sessions(MySQL_Data_Stream *myds, unsigned int n);
+	//bool move_session_to_idle_mysql_sessions(MySQL_Data_Stream *myds, unsigned int n);
 	void run_Handle_epoll_wait(int);
 #endif // IDLE_THREADS
 
-	unsigned int find_session_idx_in_mysql_sessions(MySQL_Session *sess);
-	bool set_backend_to_be_skipped_if_frontend_is_slow(MySQL_Data_Stream *myds, unsigned int n);
+	//unsigned int find_session_idx_in_mysql_sessions(MySQL_Session *sess);
+	//bool set_backend_to_be_skipped_if_frontend_is_slow(MySQL_Data_Stream *myds, unsigned int n);
 	void handle_mirror_queue_mysql_sessions();
 	void handle_kill_queues();
-	void check_timing_out_session(unsigned int n);
-	void check_for_invalid_fd(unsigned int n);
-	void read_one_byte_from_pipe(unsigned int n);
-	void tune_timeout_for_myds_needs_pause(MySQL_Data_Stream *myds);
-	void tune_timeout_for_session_needs_pause(MySQL_Data_Stream *myds);
-	void configure_pollout(MySQL_Data_Stream *myds, unsigned int n);
+	//void check_timing_out_session(unsigned int n);
+	//void check_for_invalid_fd(unsigned int n);
+	//void read_one_byte_from_pipe(unsigned int n);
+	//void tune_timeout_for_myds_needs_pause(MySQL_Data_Stream *myds);
+	//void tune_timeout_for_session_needs_pause(MySQL_Data_Stream *myds);
+	//void configure_pollout(MySQL_Data_Stream *myds, unsigned int n);
 
 	void run_MoveSessionsBetweenThreads();
 	void run_BootstrapListener();
 	int run_ComputePollTimeout();
 	void run_StopListener();
-	void run_SetAllSession_ToProcess0();
+	//void run_SetAllSession_ToProcess0();
 
 
 	protected:
@@ -157,14 +149,14 @@ class __attribute__((aligned(64))) MySQL_Thread
 
 	void *gen_args;	// this is a generic pointer to create any sort of structure
 
-	ProxySQL_Poll mypolls;
+	ProxySQL_Poll<MySQL_Data_Stream> mypolls;
 	pthread_t thread_id;
-	unsigned long long curtime;
+//	unsigned long long curtime;
 	unsigned long long pre_poll_time;
 	unsigned long long last_maintenance_time;
-	unsigned long long last_move_to_idle_thread_time;
+	//unsigned long long last_move_to_idle_thread_time;
 	std::atomic<unsigned long long> atomic_curtime;
-	PtrArray *mysql_sessions;
+	//PtrArray *mysql_sessions;
 	PtrArray *mirror_queue_mysql_sessions;
 	PtrArray *mirror_queue_mysql_sessions_cache;
 #ifdef IDLE_THREADS
@@ -175,17 +167,17 @@ class __attribute__((aligned(64))) MySQL_Thread
 #endif // IDLE_THREADS
 
 	int pipefd[2];
-	int shutdown;
+//	int shutdown;
 	kill_queue_t kq;
 
-	bool epoll_thread;
+	//bool epoll_thread;
 	bool poll_timeout_bool;
 
 	// status variables are per thread only
 	// in this way, there is no need for atomic operation and there is no cache miss
 	// when it is needed a total, all threads are checked
 	struct {
-		unsigned long long stvar[st_var_END];
+		unsigned long long stvar[MY_st_var_END];
 		unsigned int active_transactions;
 	} status_variables;
 
@@ -200,24 +192,24 @@ class __attribute__((aligned(64))) MySQL_Thread
 	pthread_mutex_t thread_mutex;
 
 	// if set_parser_algorithm == 2 , a single thr_SetParser is used
-	SetParser *thr_SetParser;
+	MySQL_Set_Stmt_Parser* thr_SetParser;
 
 	MySQL_Thread();
 	~MySQL_Thread();
-	MySQL_Session * create_new_session_and_client_data_stream(int _fd);
+	//MySQL_Session * create_new_session_and_client_data_stream(int _fd);
 	bool init();
 	void run___get_multiple_idle_connections(int& num_idles);
 	void run___cleanup_mirror_queue();
-  	void ProcessAllMyDS_BeforePoll();
-  	void ProcessAllMyDS_AfterPoll();
+  	//void ProcessAllMyDS_BeforePoll();
+  	//void ProcessAllMyDS_AfterPoll();
 	void run();
   void poll_listener_add(int sock);
   void poll_listener_del(int sock);
-  void register_session(MySQL_Session*, bool up_start=true);
+  //void register_session(MySQL_Session *, bool up_start=true);
   void unregister_session(int);
   struct pollfd * get_pollfd(unsigned int i);
   bool process_data_on_data_stream(MySQL_Data_Stream *myds, unsigned int n);
-	void ProcessAllSessions_SortingSessions();
+	//void ProcessAllSessions_SortingSessions();
 	void ProcessAllSessions_CompletedMirrorSession(unsigned int& n, MySQL_Session *sess);
 	void ProcessAllSessions_MaintenanceLoop(MySQL_Session *sess, unsigned long long sess_time, unsigned int& total_active_transactions_);
 	void ProcessAllSessions_Healthy0(MySQL_Session *sess, unsigned int& n);
@@ -287,7 +279,7 @@ struct p_th_counter {
 		mysql_unexpected_frontend_packets,
 		aws_aurora_replicas_skipped_during_query,
 		automatic_detected_sql_injection,
-		whitelisted_sqli_fingerprint,
+		mysql_whitelisted_sqli_fingerprint,
 		mysql_killed_backend_connections,
 		mysql_killed_backend_queries,
 		client_host_error_killed_connections,
@@ -541,6 +533,7 @@ class MySQL_Threads_Handler
 		int eventslog_buffer_max_query_length;
 		int eventslog_default_log;
 		int eventslog_format;
+		int eventslog_stmt_parameters;
 		char *auditlog_filename;
 		int auditlog_filesize;
 		// SSL related, proxy to server
