@@ -4,8 +4,13 @@
 #include "proxysql.h"
 #include "cpp.h"
 
-#include "../deps/json/json.hpp"
-using json = nlohmann::json;
+//#include "../deps/json/json.hpp"
+//using json = nlohmann::json;
+
+#ifndef PROXYJSON
+#define PROXYJSON
+#include "../deps/json/json_fwd.hpp"
+#endif // PROXYJSON
 
 //#define STATUS_MYSQL_CONNECTION_TRANSACTION          0x00000001 // DEPRECATED
 #define STATUS_MYSQL_CONNECTION_COMPRESSION          0x00000002
@@ -26,8 +31,8 @@ class MySQLServers_SslParams;
 class Variable {
 public:
 	char *value = (char*)"";
-	void fill_server_internal_session(json &j, int idx);
-	void fill_client_internal_session(json &j, int idx);
+	void fill_server_internal_session(nlohmann::json &j, int idx);
+	void fill_client_internal_session(nlohmann::json &j, int idx);
 };
 
 enum charset_action {
@@ -44,6 +49,7 @@ class MySQL_Connection_userinfo {
 	uint64_t hash;
 	char *username;
 	char *password;
+	PASSWORD_TYPE::E passtype;
 	char *schemaname;
 	char *sha1_pass;
 	char *fe_username;
@@ -258,13 +264,14 @@ class MySQL_Connection {
 	bool get_gtid(char *buff, uint64_t *trx_id);
 	void reduce_auto_increment_delay_token() { if (auto_increment_delay_token) auto_increment_delay_token--; };
 
+	bool match_ff_req_options(const MySQL_Connection *c);
 	bool match_tracked_options(const MySQL_Connection *c);
 	bool requires_CHANGE_USER(const MySQL_Connection *client_conn);
 	unsigned int number_of_matching_session_variables(const MySQL_Connection *client_conn, unsigned int& not_matching);
 	unsigned long get_mysql_thread_id() { return mysql ? mysql->thread_id : 0; }
 	static void set_ssl_params(MYSQL *mysql, MySQLServers_SslParams *ssl_params);
 
-	void get_mysql_info_json(json&);
-	void get_backend_conn_info_json(json&);
+	void get_mysql_info_json(nlohmann::json&);
+	void get_backend_conn_info_json(nlohmann::json&);
 };
 #endif /* __CLASS_MYSQL_CONNECTION_H */
