@@ -1,9 +1,9 @@
 #ifndef __PROXYSQL_PROMETHEUS_HELPERS_H
 #define __PROXYSQL_PROMETHEUS_HELPERS_H
 
-#include <prometheus/counter.h>
-#include <prometheus/gauge.h>
-#include <prometheus/family.h>
+#include "prometheus/counter.h"
+#include "prometheus/gauge.h"
+#include "prometheus/family.h"
 #include <string>
 
 #include "proxysql.h"
