@@ -5,7 +5,7 @@
 #include "prometheus/counter.h"
 #include "prometheus/gauge.h"
 
-#include "query_processor.h"
+//#include "query_processor.h"
 #include "proxy_defines.h"
 #include "proxysql.h"
 #include "cpp.h"
@@ -21,6 +21,11 @@ typedef struct { uint32_t hash; uint32_t key; } t_symstruct;
 class ProxySQL_Config;
 class ProxySQL_Restapi;
 
+enum SERVER_TYPE {
+	SERVER_TYPE_MYSQL,
+	SERVER_TYPE_PGSQL
+};
+
 class Scheduler_Row {
 	public:
 	unsigned int id;
@@ -93,6 +98,7 @@ struct p_admin_gauge {
 		fds_in_use,
 		version_info,
 		mysql_listener_paused,
+		pgsql_listener_paused,
 		__size
 	};
 };
@@ -186,6 +192,52 @@ struct peer_mysql_servers_v2_t {
 	peer_mysql_servers_v2_t(SQLite3_result*, const mysql_servers_v2_checksum_t&);
 };
 
+
+struct incoming_pgsql_servers_t {
+	SQLite3_result* incoming_pgsql_servers_v2 = NULL;
+	SQLite3_result* runtime_pgsql_servers = NULL;
+	SQLite3_result* incoming_replication_hostgroups = NULL;
+	SQLite3_result* incoming_hostgroup_attributes = NULL;
+
+	incoming_pgsql_servers_t();
+	incoming_pgsql_servers_t(SQLite3_result*, SQLite3_result*, SQLite3_result*, SQLite3_result*);
+};
+
+// Separate structs for runtime pgsql server and pgsql server v2 to avoid human error
+struct runtime_pgsql_servers_checksum_t {
+	std::string value;
+	time_t epoch;
+
+	runtime_pgsql_servers_checksum_t();
+	runtime_pgsql_servers_checksum_t(const std::string& value, time_t epoch);
+};
+
+struct pgsql_servers_v2_checksum_t {
+	std::string value;
+	time_t epoch;
+
+	pgsql_servers_v2_checksum_t();
+	pgsql_servers_v2_checksum_t(const std::string& value, time_t epoch);
+};
+//
+
+struct peer_runtime_pgsql_servers_t {
+	SQLite3_result* resultset{ nullptr };
+	runtime_pgsql_servers_checksum_t checksum{};
+
+	peer_runtime_pgsql_servers_t();
+	peer_runtime_pgsql_servers_t(SQLite3_result*, const runtime_pgsql_servers_checksum_t&);
+};
+
+struct peer_pgsql_servers_v2_t {
+	SQLite3_result* resultset{ nullptr };
+	pgsql_servers_v2_checksum_t checksum{};
+
+	peer_pgsql_servers_v2_t();
+	peer_pgsql_servers_v2_t(SQLite3_result*, const pgsql_servers_v2_checksum_t&);
+};
+
+
 class ProxySQL_Admin {
 	private:
 	volatile int main_shutdown;
@@ -214,6 +266,12 @@ class ProxySQL_Admin {
 	rwlock_t mysql_servers_rwlock;
 #endif
 
+#ifdef PA_PTHREAD_MUTEX
+	pthread_mutex_t pgsql_servers_lock;
+#else
+	rwlock_t pgsql_servers_rwlock;
+#endif
+
 	prometheus::SerialExposer serial_exposer;
 
 	std::mutex proxysql_servers_mutex;
@@ -226,6 +284,7 @@ class ProxySQL_Admin {
 		char *stats_credentials;
 		int refresh_interval;
 		char *mysql_ifaces;
+		char *pgsql_ifaces;
 		char *telnet_admin_ifaces;
 		char *telnet_stats_ifaces;
 		bool admin_read_only;
@@ -259,6 +318,7 @@ class ProxySQL_Admin {
 		int stats_system_cpu;
 		int stats_system_memory;
 		int mysql_show_processlist_extended;
+		int pgsql_show_processlist_extended;
 		bool restapi_enabled;
 		bool restapi_enabled_old;
 		int restapi_port;
@@ -332,8 +392,11 @@ class ProxySQL_Admin {
 	 *  param 'resultset' is supplied, it will match it's value, otherwise it will be a locally created
 	 *  'SQLite3_result*' that should be freed.
 	 */
+	template<enum SERVER_TYPE>
 	SQLite3_result* __add_active_users(enum cred_username_type usertype, char *user=NULL, SQLite3_result* resultset = nullptr);
+	template<enum SERVER_TYPE>
 	void __delete_inactive_users(enum cred_username_type usertype);
+	template<enum SERVER_TYPE>
 	void add_admin_users();
 	void __refresh_users(std::unique_ptr<SQLite3_result>&& all_users = nullptr, const std::string& checksum = "", const time_t epoch = 0);
 	void __add_active_users_ldap();
@@ -364,10 +427,14 @@ class ProxySQL_Admin {
 	void disk_upgrade_rest_api_routes();
 
 #ifdef DEBUG
+	template<enum SERVER_TYPE>
 	void add_credentials(char *type, char *credentials, int hostgroup_id);
+	template<enum SERVER_TYPE>
 	void delete_credentials(char *type, char *credentials);
 #else
+	template<enum SERVER_TYPE>
 	void add_credentials(char *credentials, int hostgroup_id);
+	template<enum SERVER_TYPE>
 	void delete_credentials(char *credentials);
 #endif /* DEBUG */
 
@@ -380,6 +447,14 @@ class ProxySQL_Admin {
 	void flush_clickhouse_variables___database_to_runtime(SQLite3DB *db, bool replace);
 #endif /* PROXYSQLCLICKHOUSE */
 
+	// PostgreSQL
+	void __refresh_pgsql_users(std::unique_ptr<SQLite3_result>&& pgsql_users_resultset = nullptr, const std::string& checksum = "", const time_t epoch = 0);
+	//void __add_active_pgsql_users(char* user = NULL);
+	//void __delete_inactive_pgsql_users();
+	void flush_pgsql_variables___runtime_to_database(SQLite3DB* db, bool replace, bool del, bool onlyifempty, bool runtime = false, bool use_lock = true);
+	void flush_pgsql_variables___database_to_runtime(SQLite3DB* db, bool replace, const std::string& checksum = "", const time_t epoch = 0);
+	//
+
 	void flush_sqliteserver_variables___runtime_to_database(SQLite3DB *db, bool replace, bool del, bool onlyifempty, bool runtime=false);
 	void flush_sqliteserver_variables___database_to_runtime(SQLite3DB *db, bool replace);
 	
@@ -406,8 +481,9 @@ class ProxySQL_Admin {
 		bool checksum_admin_variables;
 		bool checksum_ldap_variables;
 	} checksum_variables;
+	template<enum SERVER_TYPE pt>
 	void public_add_active_users(enum cred_username_type usertype, char *user=NULL) {
-		__add_active_users(usertype, user);
+		__add_active_users<pt>(usertype, user);
 	}
 	// @brief True if all ProxySQL modules have been already started. End of 'phase3'.
 	bool all_modules_started;
@@ -481,8 +557,11 @@ class ProxySQL_Admin {
 	void save_mysql_servers_runtime_to_database(bool _runtime);
 	void admin_shutdown();
 	bool is_command(std::string);
-	void send_MySQL_OK(MySQL_Protocol *myprot, char *msg, int rows=0);
-	void send_MySQL_ERR(MySQL_Protocol *myprot, char *msg, uint32_t code=1045);
+
+	template <typename S>
+	void send_ok_msg_to_client(S* sess, const char *msg, int rows, const char* query);
+	template <typename S>
+	void send_error_msg_to_client(S* sess, const char *msg, uint16_t mysql_err_code=1045);
 #ifdef DEBUG
 	// these two following functions used to just call and return one function each
 	// this approach was replaced when we introduced debug filters
@@ -532,12 +611,14 @@ class ProxySQL_Admin {
 	char* load_mysql_query_rules_to_runtime(SQLite3_result* SQLite3_query_rules_resultset=NULL, SQLite3_result* SQLite3_query_rules_fast_routing_resultset=NULL, const std::string& checksum = "", const time_t epoch = 0);
 	void save_mysql_query_rules_from_runtime(bool);
 	void save_mysql_query_rules_fast_routing_from_runtime(bool);
-	char * load_mysql_firewall_to_runtime();
+	char* load_mysql_firewall_to_runtime();
 	void save_mysql_firewall_from_runtime(bool);
 	void save_mysql_firewall_whitelist_users_from_runtime(bool, SQLite3_result *);
 	void save_mysql_firewall_whitelist_rules_from_runtime(bool, SQLite3_result *);
 	void save_mysql_firewall_whitelist_sqli_fingerprints_from_runtime(bool, SQLite3_result *);
 
+	char* load_pgsql_firewall_to_runtime();
+
 	void load_scheduler_to_runtime();
 	void save_scheduler_runtime_to_database(bool);
 
@@ -560,6 +641,7 @@ class ProxySQL_Admin {
 	);
 	int stats___mysql_query_digests(bool reset, bool copy=false);
 	int stats___mysql_query_digests_v2(bool reset, bool copy, bool use_resultset);
+	int stats___pgsql_query_digests_v2(bool reset, bool copy, bool use_resultset);
 	//void stats___mysql_query_digests_reset();
 	void stats___mysql_commands_counters();
 	void stats___mysql_processlist();
@@ -570,6 +652,20 @@ class ProxySQL_Admin {
 	void stats___mysql_global();
 	void stats___mysql_users();
 
+	void stats___pgsql_global();
+	void stats___pgsql_users();
+	void stats___pgsql_free_connections();
+	void stats___pgsql_connection_pool(bool _reset);
+	void stats___pgsql_processlist();
+	void stats___pgsql_errors(bool reset);
+	void stats___pgsql_client_host_cache(bool reset);
+	void stats___pgsql_query_rules();
+	void stats___pgsql_commands_counters();
+	int  stats___save_pgsql_query_digest_to_sqlite(
+		const bool reset, const bool copy, const SQLite3_result* resultset,
+		const umap_query_digest* digest_umap, const umap_query_digest_text* digest_text_umap
+	);
+
 	void stats___proxysql_servers_checksums();
 	void stats___proxysql_servers_metrics();
 	void stats___proxysql_message_metrics(bool reset);
@@ -592,6 +688,9 @@ class ProxySQL_Admin {
 	void mysql_servers_wrlock();
 	void mysql_servers_wrunlock();
 
+	void pgsql_servers_wrlock();
+	void pgsql_servers_wrunlock();
+
 	char *get_variable(char *name);
 
 	// wrapper to call a private function
@@ -629,10 +728,47 @@ class ProxySQL_Admin {
 	void save_clickhouse_users_runtime_to_database(bool _runtime);
 #endif /* PROXYSQLCLICKHOUSE */
 
+	//PostgreSQL
+	void init_pgsql_servers();
+	void init_pgsql_query_rules();
+	void init_pgsql_firewall();
+
+	void init_pgsql_variables();
+	void load_pgsql_variables_to_runtime(const std::string& checksum = "", const time_t epoch = 0) { flush_pgsql_variables___database_to_runtime(admindb, true, checksum, epoch); }
+	void save_pgsql_variables_from_runtime() { flush_pgsql_variables___runtime_to_database(admindb, true, true, false); }
+	void init_pgsql_users(std::unique_ptr<SQLite3_result>&& pgsql_users_resultset = nullptr, const std::string& checksum = "", const time_t epoch = 0);
+	void flush_pgsql_users__from_memory_to_disk();
+	void flush_pgsql_users__from_disk_to_memory();
+
+	void save_pgsql_users_runtime_to_database(bool _runtime);
+
+	void load_pgsql_servers_to_runtime(const incoming_pgsql_servers_t& incoming_pgsql_servers = {}, const runtime_pgsql_servers_checksum_t& peer_runtime_pgsql_server = {},
+		const pgsql_servers_v2_checksum_t& peer_pgsql_server_v2 = {});
+
+	char* load_pgsql_query_rules_to_runtime(SQLite3_result* SQLite3_query_rules_resultset = NULL, 
+		SQLite3_result* SQLite3_query_rules_fast_routing_resultset = NULL, const std::string& checksum = "", const time_t epoch = 0);
+
+	void save_pgsql_servers_runtime_to_database(bool _runtime);
+	void save_pgsql_firewall_from_runtime(bool);
+	void save_pgsql_query_rules_from_runtime(bool);
+	void save_pgsql_query_rules_fast_routing_from_runtime(bool);
+	void save_pgsql_firewall_whitelist_users_from_runtime(bool, SQLite3_result*);
+	void save_pgsql_firewall_whitelist_rules_from_runtime(bool, SQLite3_result*);
+	void save_pgsql_firewall_whitelist_sqli_fingerprints_from_runtime(bool, SQLite3_result*);
+
+	void save_pgsql_ldap_mapping_runtime_to_database(bool);
+	//
+
 	void vacuum_stats(bool);
+
+	template <enum SERVER_TYPE pt>
 	int FlushDigestTableToDisk(SQLite3DB *);
 
 	bool ProxySQL_Test___Load_MySQL_Whitelist(int *, int *, int, int);
+	void map_test_mysql_firewall_whitelist_rules_cleanup();
+
+	template<typename S>
+	void ProxySQL_Test_Handler(ProxySQL_Admin *SPA, S* sess, char *query_no_space, bool& run_query);
 
 
 #ifdef TEST_AURORA
@@ -666,7 +802,8 @@ class ProxySQL_Admin {
 	unsigned long long ProxySQL_Test___MySQL_HostGroups_Manager_Balancing_HG5211();
 	bool ProxySQL_Test___CA_Certificate_Load_And_Verify(uint64_t* duration, int cnt, const char* cacert, const char* capath);
 #endif
-	friend void admin_session_handler(MySQL_Session *sess, void *_pa, PtrSize_t *pkt);
+	template<typename S>
+	friend void admin_session_handler(S* sess, void *_pa, PtrSize_t *pkt);
 
 	// FLUSH LOGS
 	void flush_logs();
