@@ -16,6 +16,9 @@ public:
 	int Read_MySQL_Users_from_configfile();
 	int Read_MySQL_Query_Rules_from_configfile();
 	int Read_MySQL_Servers_from_configfile();
+	int Read_PgSQL_Servers_from_configfile();
+	int Read_PgSQL_Users_from_configfile();
+	int Read_PgSQL_Query_Rules_from_configfile();
 	int Read_Scheduler_from_configfile();
 	int Read_Restapi_from_configfile();
 	int Read_ProxySQL_Servers_from_configfile();
@@ -25,6 +28,9 @@ public:
 	int Write_MySQL_Users_to_configfile(std::string& data);
 	int Write_MySQL_Query_Rules_to_configfile(std::string& data);
 	int Write_MySQL_Servers_to_configfile(std::string& data);
+	int Write_PgSQL_Servers_to_configfile(std::string& data);
+	int Write_PgSQL_Users_to_configfile(std::string& data);
+	int Write_PgSQL_Query_Rules_to_configfile(std::string& data);
 	int Write_Scheduler_to_configfile(std::string& data);
 	int Write_Restapi_to_configfile(std::string& data);
 	int Write_ProxySQL_Servers_to_configfile(std::string& data);
