@@ -5,11 +5,12 @@
 #include "re2/re2.h"
 #include "re2/regexp.h"
 #include "proxysql.h"
+#include "clickhouse/client.h"
 #include "cpp.h"
 
 #include "MySQL_Logger.hpp"
 #include "MySQL_Data_Stream.h"
-#include "query_processor.h"
+#include "MySQL_Query_Processor.h"
 
 #include <search.h>
 #include <stdlib.h>
@@ -440,10 +441,10 @@ static char *s_strdup(char *s) {
 
 
 static int __ClickHouse_Server_refresh_interval=1000;
-extern Query_Cache *GloQC;
+extern MySQL_Query_Cache *GloMyQC;
 extern ClickHouse_Authentication *GloClickHouseAuth;
 extern ProxySQL_Admin *GloAdmin;
-extern Query_Processor *GloQPro;
+extern MySQL_Query_Processor* GloMyQPro;
 extern MySQL_Threads_Handler *GloMTH;
 extern MySQL_Logger *GloMyLogger;
 extern MySQL_Monitor *GloMyMon;
@@ -585,7 +586,7 @@ class sqlite3server_main_loop_listeners {
 
 static sqlite3server_main_loop_listeners S_amll;
 
-void ClickHouse_Server_session_handler(MySQL_Session *sess, void *_pa, PtrSize_t *pkt) {
+void ClickHouse_Server_session_handler(MySQL_Session* sess, void *_pa, PtrSize_t *pkt) {
 	char *error=NULL;
 	int cols;
 	int affected_rows;
@@ -612,7 +613,7 @@ void ClickHouse_Server_session_handler(MySQL_Session *sess, void *_pa, PtrSize_t
 		query_no_space[query_no_space_length]=0;
 	}
 
-	proxy_debug(PROXY_DEBUG_SQLITE, 4, "Received query on Session %p , thread_session_id %u : %s\n", sess, sess->thread_session_id, query_no_space);
+	proxy_debug(PROXY_DEBUG_SQLITE, 4, "Received query on Session %p , thread_session_id %u : %s\n", (MySQL_Session*)sess, sess->thread_session_id, query_no_space);
 
 
 	if (sess->session_type == PROXYSQL_SESSION_CLICKHOUSE) {
@@ -1145,6 +1146,8 @@ __end_show_commands:
 			||
 			(strncasecmp("SHOW SLAVE STATUS",query_no_space,17)==0)
 			||
+			(strncasecmp("SHOW REPLICA STATUS",query_no_space,19)==0)
+			||
 			(strncasecmp("SHOW MASTER LOGS",query_no_space,16)==0)
 		) {
 			GloClickHouseServer->send_MySQL_ERR(&sess->client_myds->myprot, (char *)"Access Denied");
@@ -1340,6 +1343,7 @@ bool ClickHouse_Session::init() {
 	hostname = GloClickHouseServer->get_variable((char *)"hostname");
 	port = GloClickHouseServer->get_variable((char *)"port");
 	try {
+		clickhouse::ClientOptions co;
 		co.SetHost(hostname);
 		co.SetPort(atoi(port));
 		co.SetCompressionMethod(CompressionMethod::None);
@@ -1393,9 +1397,9 @@ static void *child_mysql(void *arg) {
 	sqlite_sess->init();
 	mysql_thr->gen_args = (void *)sqlite_sess;
 
-	GloQPro->init_thread();
+	GloMyQPro->init_thread();
 	mysql_thr->refresh_variables();
-	sess=mysql_thr->create_new_session_and_client_data_stream(client);
+	sess=mysql_thr->create_new_session_and_client_data_stream<MySQL_Thread, MySQL_Session*>(client);
 	sess->thread=mysql_thr;
 	sess->session_type = PROXYSQL_SESSION_CLICKHOUSE;
 	sess->handler_function=ClickHouse_Server_session_handler;
@@ -1475,7 +1479,7 @@ static void * sqlite3server_main_loop(void *arg)
 	pthread_attr_t attr;
 	pthread_attr_init(&attr);
 	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
-	set_thread_name("ClickHouseMain");
+	set_thread_name("ClickHouseMain", GloVars.set_thread_name);
 	while (glovars.shutdown==0 && *shutdown==0)
 	{
 		int *client;
