@@ -114,6 +114,10 @@ ProxySQL_GlobalVariables::~ProxySQL_GlobalVariables() {
 	checksums_values.mysql_variables.in_shutdown = true;
 	checksums_values.ldap_variables.in_shutdown = true;
 	checksums_values.proxysql_servers.in_shutdown = true;
+	checksums_values.pgsql_query_rules.in_shutdown = true;
+	checksums_values.pgsql_servers.in_shutdown = true;
+	checksums_values.pgsql_users.in_shutdown = true;
+	checksums_values.pgsql_variables.in_shutdown = true;
 	if (global.gr_bootstrap_uri) {
 		free(global.gr_bootstrap_uri);
 		global.gr_bootstrap_uri = nullptr;
@@ -193,7 +197,8 @@ ProxySQL_GlobalVariables::ProxySQL_GlobalVariables() :
 	global.gdbg=false;
 	global.nostart=false;
 	global.foreground=false;
-	global.monitor=true;
+	global.my_monitor=true;
+	global.pg_monitor=true;
 #ifdef IDLE_THREADS
 	global.idle_threads=false;
 #endif /* IDLE_THREADS */
@@ -477,7 +482,8 @@ void ProxySQL_GlobalVariables::process_opts_post() {
 	}
 
 	if (opt->isSet("-M")) {
-		global.monitor=false;
+		global.my_monitor=false;
+		global.pg_monitor=false;
 	}
 
 #ifdef SO_REUSEPORT
@@ -578,6 +584,31 @@ uint64_t ProxySQL_GlobalVariables::generate_global_checksum() {
 		myhash.Update(v->checksum,strlen(v->checksum));
 		myhash.Update(&v->version,sizeof(v->version));
 	}
+	v = &checksums_values.pgsql_query_rules;
+	if (v->version) {
+		myhash.Update(v->checksum, strlen(v->checksum));
+		myhash.Update(&v->version, sizeof(v->version));
+	}
+	v = &checksums_values.pgsql_servers;
+	if (v->version) {
+		myhash.Update(v->checksum, strlen(v->checksum));
+		myhash.Update(&v->version, sizeof(v->version));
+	}
+	v = &checksums_values.pgsql_servers_v2;
+	if (v->version) {
+		myhash.Update(v->checksum, strlen(v->checksum));
+		myhash.Update(&v->version, sizeof(v->version));
+	}
+	v = &checksums_values.pgsql_users;
+	if (v->version) {
+		myhash.Update(v->checksum, strlen(v->checksum));
+		myhash.Update(&v->version, sizeof(v->version));
+	}
+	v = &checksums_values.pgsql_variables;
+	if (v->version) {
+		myhash.Update(v->checksum, strlen(v->checksum));
+		myhash.Update(&v->version, sizeof(v->version));
+	}
 	v = &checksums_values.proxysql_servers;
 	if (v->version) {
 		myhash.Update(v->checksum,strlen(v->checksum));
