@@ -3,12 +3,12 @@
 #include <string.h>
 #include "StatCounters.h"
 #include "MySQL_Data_Stream.h"
+#include "PgSQL_Data_Stream.h"
 #include "ProxySQL_Poll.h"
 #include "proxysql_structs.h"
 #include <poll.h>
 #include "cpp.h"
 
-
 /**
  * @file ProxySQL_Poll.cpp
  *
@@ -24,10 +24,11 @@
  * This function reduces the size of the ProxySQL_Poll object by reallocating memory to fit the current number of elements.
  * It adjusts the size of internal arrays to a size that is a power of two near the current number of elements.
  */
-void ProxySQL_Poll::shrink() {
+ template<class T>
+void ProxySQL_Poll<T>::shrink() {
 	unsigned int new_size=l_near_pow_2(len+1);
 	fds=(struct pollfd *)realloc(fds,new_size*sizeof(struct pollfd));
-	myds=(MySQL_Data_Stream **)realloc(myds,new_size*sizeof(MySQL_Data_Stream *));
+	myds=(T **)realloc(myds,new_size*sizeof(T *));
 	last_recv=(unsigned long long *)realloc(last_recv,new_size*sizeof(unsigned long long));
 	last_sent=(unsigned long long *)realloc(last_sent,new_size*sizeof(unsigned long long));
 	size=new_size;
@@ -41,23 +42,26 @@ void ProxySQL_Poll::shrink() {
  * 
  * @param more The number of additional elements to accommodate.
  */
-void ProxySQL_Poll::expand(unsigned int more) {
+ template<class T>
+void ProxySQL_Poll<T>::expand(unsigned int more) {
 	if ( (len+more) > size ) {
 		unsigned int new_size=l_near_pow_2(len+more);
 		fds=(struct pollfd *)realloc(fds,new_size*sizeof(struct pollfd));
-		myds=(MySQL_Data_Stream **)realloc(myds,new_size*sizeof(MySQL_Data_Stream *));
+		myds=(T **)realloc(myds,new_size*sizeof(T *));
 		last_recv=(unsigned long long *)realloc(last_recv,new_size*sizeof(unsigned long long));
 		last_sent=(unsigned long long *)realloc(last_sent,new_size*sizeof(unsigned long long));
 		size=new_size;
 	}
 }
 
+
 /**
  * @brief Constructs a new ProxySQL_Poll object.
  * 
  * This constructor initializes a new ProxySQL_Poll object with default values and allocates memory for internal arrays.
  */
-ProxySQL_Poll::ProxySQL_Poll() {
+template<class T>
+ProxySQL_Poll<T>::ProxySQL_Poll() {
 	loop_counters=new StatCounters(15,10);
 	poll_timeout=0;
 	loops=0;
@@ -66,7 +70,7 @@ ProxySQL_Poll::ProxySQL_Poll() {
 	pending_listener_del=0;
 	size=MIN_POLL_LEN;
 	fds=(struct pollfd *)malloc(size*sizeof(struct pollfd));
-	myds=(MySQL_Data_Stream **)malloc(size*sizeof(MySQL_Data_Stream *));
+	myds=(T**)malloc(size*sizeof(T *));
 	last_recv=(unsigned long long *)malloc(size*sizeof(unsigned long long));
 	last_sent=(unsigned long long *)malloc(size*sizeof(unsigned long long));
 }
@@ -76,7 +80,8 @@ ProxySQL_Poll::ProxySQL_Poll() {
  * 
  * This destructor deallocates memory for internal arrays and releases resources associated with the ProxySQL_Poll object.
  */
-ProxySQL_Poll::~ProxySQL_Poll() {
+template<class T>
+ProxySQL_Poll<T>::~ProxySQL_Poll() {
 	unsigned int i;
 	for (i=0;i<len;i++) {
 		if (
@@ -103,7 +108,8 @@ ProxySQL_Poll::~ProxySQL_Poll() {
  * @param _myds The MySQL_Data_Stream associated with the FD.
  * @param sent_time The time when data was last sent on the FD.
  */
-void ProxySQL_Poll::add(uint32_t _events, int _fd, MySQL_Data_Stream *_myds, unsigned long long sent_time) {
+template<class T>
+void ProxySQL_Poll<T>::add(uint32_t _events, int _fd, T *_myds, unsigned long long sent_time) {
 	if (len==size) {
 		expand(1);
 	}
@@ -120,6 +126,21 @@ void ProxySQL_Poll::add(uint32_t _events, int _fd, MySQL_Data_Stream *_myds, uns
 	len++;
 }
 
+/**
+ * @brief Updates the file descriptor (FD) at a specific index in the ProxySQL_Poll object.
+ *
+ * This function updates the file descriptor (FD) at a specific index in the ProxySQL_Poll object.
+ * It does not modify any other associated data or metadata.
+ *
+ * @param idx The index of the file descriptor (FD) to update.
+ * @param _fd The new file descriptor (FD) value.
+ */
+template<class T>
+void ProxySQL_Poll<T>::update_fd_at_index(unsigned int idx, int _fd) {
+	if ((int)idx == -1 || idx >= len) return;
+	fds[idx].fd = _fd;
+}
+
 /**
  * @brief Removes a file descriptor (FD) and its associated MySQL_Data_Stream from the ProxySQL_Poll object.
  * 
@@ -128,7 +149,8 @@ void ProxySQL_Poll::add(uint32_t _events, int _fd, MySQL_Data_Stream *_myds, uns
  * 
  * @param i The index of the file descriptor (FD) to remove.
  */
-void ProxySQL_Poll::remove_index_fast(unsigned int i) {
+template<class T>
+void ProxySQL_Poll<T>::remove_index_fast(unsigned int i) {
 	if ((int)i==-1) return;
 	myds[i]->poll_fds_idx=-1; // this prevents further delete
 	if (i != (len-1)) {
@@ -155,7 +177,8 @@ void ProxySQL_Poll::remove_index_fast(unsigned int i) {
  * @param fd The file descriptor (FD) to search for.
  * @return The index of the file descriptor (FD) if found, otherwise -1.
  */
-int ProxySQL_Poll::find_index(int fd) {
+template<class T>
+int ProxySQL_Poll<T>::find_index(int fd) {
 	unsigned int i;
 	for (i=0; i<len; i++) {
 		if (fds[i].fd==fd) {
@@ -164,3 +187,6 @@ int ProxySQL_Poll::find_index(int fd) {
 	}
 	return -1;
 }
+
+template class ProxySQL_Poll<PgSQL_Data_Stream>;
+template class ProxySQL_Poll<MySQL_Data_Stream>;
