@@ -1,3 +1,7 @@
+#include "../deps/json/json.hpp"
+using json = nlohmann::json;
+#define PROXYJSON
+
 #include "proxysql.h"
 #include "cpp.h"
 #include "httpserver.hpp"
@@ -17,7 +21,7 @@
 extern ProxySQL_Admin *GloAdmin;
 
 using namespace httpserver;
-using nlohmann::json;
+//using nlohmann::json;
 
 class sync_resource : public http_resource {
 private:
@@ -329,7 +333,7 @@ public:
 };
 
 void * restapi_server_thread(void *arg) {
-	set_thread_name("RESTAPI_Server");
+	set_thread_name("RESTAPI_Server", GloVars.set_thread_name);
 	httpserver::webserver * ws = (httpserver::webserver *)arg;
     ws->start(true);
 	return NULL;
