@@ -15,36 +15,71 @@ using std::string;
 using std::unordered_map;
 
 #ifdef DEBUG
-#ifdef DEBUG_EXTERN
-#undef DEBUG_EXTERN
-#endif /* DEBUG_EXTERN */
-#endif /* DEBUG */
 
-#ifndef CLOCK_MONOTONIC
-#define CLOCK_MONOTONIC SYSTEM_CLOCK
-#endif // CLOCK_MONOTONIC
-
-#ifdef DEBUG
 __thread unsigned long long pretime=0;
 static pthread_mutex_t debug_mutex;
 static pthread_rwlock_t filters_rwlock;
 static SQLite3DB * debugdb_disk = NULL;
 sqlite3_stmt *statement1=NULL;
 static unsigned int debug_output = 1;
-#endif /* DEBUG */
 
-/*
-static inline unsigned long long debug_monotonic_time() {
-	struct timespec ts;
-	clock_gettime(CLOCK_MONOTONIC, &ts);
-	return (((unsigned long long) ts.tv_sec) * 1000000) + (ts.tv_nsec / 1000);
-}
-*/
+
+
 
 #define DEBUG_MSG_MAXSIZE	1024
 
-#ifdef DEBUG
 
+struct DebugLogEntry {
+	unsigned long long time;
+	unsigned long long lapse;
+	int thr;
+	std::string file;
+	int line;
+	std::string funct;
+	int module;
+	std::string modname;
+	int verbosity;
+	std::string message;
+	std::string backtrace;
+};
+
+static const size_t limitSize = 100;
+static std::vector<DebugLogEntry> log_buffer = {};
+
+
+/**
+ * @brief Synchronizes the log buffer to the SQLite database.
+ *
+ * This function writes the contents of the shared log buffer to the SQLite database
+ * in a single transaction. It assumes that the log buffer is protected by a mutex
+ * and that the prepared statement `statement1` has already been prepared.
+ * This happens in proxy_debug_func()
+ *
+ * @param db A pointer to the SQLite3DB object.
+ */
+void sync_log_buffer_to_disk(SQLite3DB *db) {
+	int rc;
+	db->execute("BEGIN TRANSACTION");
+	for (const auto& entry : log_buffer) {
+		rc=(*proxy_sqlite3_bind_int64)(statement1, 1, entry.time); ASSERT_SQLITE_OK(rc, db);
+		rc=(*proxy_sqlite3_bind_int64)(statement1, 2, entry.lapse);ASSERT_SQLITE_OK(rc, db);
+		rc=(*proxy_sqlite3_bind_int64)(statement1, 3, entry.thr); ASSERT_SQLITE_OK(rc, db);
+		rc=(*proxy_sqlite3_bind_text)(statement1, 4, entry.file.c_str(), -1, SQLITE_TRANSIENT); ASSERT_SQLITE_OK(rc, db);
+		rc=(*proxy_sqlite3_bind_int64)(statement1, 5, entry.line); ASSERT_SQLITE_OK(rc, db);
+		rc=(*proxy_sqlite3_bind_text)(statement1, 6, entry.funct.c_str(), -1, SQLITE_TRANSIENT); ASSERT_SQLITE_OK(rc, db);
+		rc=(*proxy_sqlite3_bind_int64)(statement1, 7, entry.module); ASSERT_SQLITE_OK(rc, db);
+		rc=(*proxy_sqlite3_bind_text)(statement1, 8, entry.modname.c_str(), -1, SQLITE_TRANSIENT); ASSERT_SQLITE_OK(rc, db);
+		rc=(*proxy_sqlite3_bind_int64)(statement1, 9, entry.verbosity); ASSERT_SQLITE_OK(rc, db);
+		rc=(*proxy_sqlite3_bind_text)(statement1, 10, entry.message.c_str(), -1, SQLITE_TRANSIENT); ASSERT_SQLITE_OK(rc, db);
+		rc=(*proxy_sqlite3_bind_text)(statement1, 11, entry.backtrace.c_str(), -1, SQLITE_TRANSIENT); ASSERT_SQLITE_OK(rc, db);
+		SAFE_SQLITE3_STEP2(statement1);
+		rc=(*proxy_sqlite3_clear_bindings)(statement1); ASSERT_SQLITE_OK(rc, db);
+		// Note: no assert() in proxy_debug_func() after sqlite3_reset() because it is possible that we are in shutdown
+		rc=(*proxy_sqlite3_reset)(statement1); // ASSERT_SQLITE_OK(rc, db);
+	}
+	db->execute("COMMIT");
+	log_buffer.clear();
+}
 
 /**
  * @brief Contains all filters related to debug.
@@ -58,7 +93,6 @@ static inline unsigned long long debug_monotonic_time() {
 std::set<std::string>* debug_filters = nullptr;
 
 static bool filter_debug_entry(const char *__file, int __line, const char *__func) {
-	//pthread_mutex_lock(&debug_mutex);
 	pthread_rwlock_rdlock(&filters_rwlock);
 	bool to_filter = false;
 	if (debug_filters && debug_filters->size()) { // if the set is empty we aren't performing any filter, so we won't search
@@ -99,7 +133,6 @@ static bool filter_debug_entry(const char *__file, int __line, const char *__fun
 			}
 		}
 	}
-	//pthread_mutex_unlock(&debug_mutex);
 	pthread_rwlock_unlock(&filters_rwlock);
 	return to_filter;
 }
@@ -107,19 +140,16 @@ static bool filter_debug_entry(const char *__file, int __line, const char *__fun
 // we use this function to sent the filters to Admin
 // we hold here the lock on filters_rwlock
 void proxy_debug_get_filters(std::set<std::string>& f) {
-	//pthread_mutex_lock(&debug_mutex);
 	pthread_rwlock_rdlock(&filters_rwlock);
 	if (debug_filters) {
 		f = *debug_filters;
 	}
 	pthread_rwlock_unlock(&filters_rwlock);
-	//pthread_mutex_unlock(&debug_mutex);
 }
 
 // we use this function to get the filters from Admin
 // we hold here the lock on filters_rwlock
 void proxy_debug_load_filters(std::set<std::string>& f) {
-	//pthread_mutex_lock(&debug_mutex);
 	pthread_rwlock_wrlock(&filters_rwlock);
 	if (debug_filters) {
 		debug_filters->erase(debug_filters->begin(), debug_filters->end());
@@ -128,11 +158,19 @@ void proxy_debug_load_filters(std::set<std::string>& f) {
 		debug_filters = new std::set<std::string>(f);
 	}
 	pthread_rwlock_unlock(&filters_rwlock);
-	//pthread_mutex_unlock(&debug_mutex);
 }
 
 // REMINDER: This function should always save/restore 'errno', otherwise it could influence error handling.
-void proxy_debug_func(enum debug_module module, int verbosity, int thr, const char *__file, int __line, const char *__func, const char *fmt, ...) {
+void proxy_debug_func(
+	enum debug_module module,
+	int verbosity,
+	int thr,
+	const char *__file,
+	int __line,
+	const char *__func,
+	const char *fmt,
+	...
+) {
 	int saved_errno = errno;
 	assert(module<PROXY_DEBUG_UNKNOWN);
 	if (pretime == 0) { // never initialized
@@ -147,17 +185,22 @@ void proxy_debug_func(enum debug_module module, int verbosity, int thr, const ch
 		errno = saved_errno;
 		return;
 	}
+
 	char origdebugbuff[DEBUG_MSG_MAXSIZE];
 	char debugbuff[DEBUG_MSG_MAXSIZE];
 	char longdebugbuff[DEBUG_MSG_MAXSIZE*8];
 	char longdebugbuff2[DEBUG_MSG_MAXSIZE*8];
+
 	longdebugbuff[0]=0;
 	longdebugbuff2[0]=0;
+
 	unsigned long long curtime=realtime_time();
 	bool write_to_disk = false;
+
 	if (debugdb_disk != NULL && (debug_output == 2 || debug_output == 3)) {
 		write_to_disk = true;
 	}
+
 	if (
 		GloVars.global.foreground
 		||
@@ -167,7 +210,6 @@ void proxy_debug_func(enum debug_module module, int verbosity, int thr, const ch
 		va_start(ap, fmt);
 		vsnprintf(origdebugbuff, DEBUG_MSG_MAXSIZE,fmt,ap);
 		va_end(ap);
-		//fprintf(stderr, "%d:%s:%d:%s(): MOD#%d LVL#%d : %s" , thr, __file, __line, __func, module, verbosity, debugbuff);
 		sprintf(longdebugbuff, "%llu(%llu): %d:%s:%d:%s(): MOD#%d#%s LVL#%d : %s" , curtime, curtime-pretime, thr, __file, __line, __func, module, GloVars.global.gdbg_lvl[module].name, verbosity, origdebugbuff);
 	}
 #ifdef __GLIBC__
@@ -176,14 +218,12 @@ void proxy_debug_func(enum debug_module module, int verbosity, int thr, const ch
 		char **strings;
 		int s;
 		s = backtrace(arr, 20);
-		//backtrace_symbols_fd(arr, s, STDERR_FILENO);
 		strings=backtrace_symbols(arr,s);
 		if (strings == NULL) {
 			perror("backtrace_symbols");
 			exit(EXIT_FAILURE);
 		}
 		for (int i=0; i<s; i++) {
-			//printf("%s\n", strings[i]);
 			debugbuff[0]=0;
 			sscanf(strings[i], "%*[^(](%100[^+]", debugbuff);
 			int status;
@@ -194,11 +234,7 @@ void proxy_debug_func(enum debug_module module, int verbosity, int thr, const ch
 				strcat(longdebugbuff2,debugbuff);
 			}
 		}
-		//printf("\n");
-		//strcat(longdebugbuff2,"\n");
 		free(strings);
-//	} else {
-//		fprintf(stderr, "%s", longdebugbuff);
 	}
 #endif
 	pthread_mutex_lock(&debug_mutex);
@@ -230,21 +266,32 @@ void proxy_debug_func(enum debug_module module, int verbosity, int thr, const ch
 			}
 		}
 		if (write_to_disk == true) {
-			rc=(*proxy_sqlite3_bind_int64)(statement1, 1, curtime); ASSERT_SQLITE_OK(rc, db);
-			rc=(*proxy_sqlite3_bind_int64)(statement1, 2, curtime-pretime); ASSERT_SQLITE_OK(rc, db);
-			rc=(*proxy_sqlite3_bind_int64)(statement1, 3, thr); ASSERT_SQLITE_OK(rc, db);
-			rc=(*proxy_sqlite3_bind_text)(statement1,  4, __file, -1, SQLITE_TRANSIENT); ASSERT_SQLITE_OK(rc, db);
-			rc=(*proxy_sqlite3_bind_int64)(statement1, 5, __line); ASSERT_SQLITE_OK(rc, db);
-			rc=(*proxy_sqlite3_bind_text)(statement1,  6, __func, -1, SQLITE_TRANSIENT); ASSERT_SQLITE_OK(rc, db);
-			rc=(*proxy_sqlite3_bind_int64)(statement1, 7, module); ASSERT_SQLITE_OK(rc, db);
-			rc=(*proxy_sqlite3_bind_text)(statement1,  8, GloVars.global.gdbg_lvl[module].name, -1, SQLITE_TRANSIENT); ASSERT_SQLITE_OK(rc, db);
-			rc=(*proxy_sqlite3_bind_int64)(statement1, 9, verbosity); ASSERT_SQLITE_OK(rc, db);
-			rc=(*proxy_sqlite3_bind_text)(statement1, 10, origdebugbuff, -1, SQLITE_TRANSIENT); ASSERT_SQLITE_OK(rc, db);
-			rc=(*proxy_sqlite3_bind_text)(statement1, 11, longdebugbuff2, -1, SQLITE_TRANSIENT); ASSERT_SQLITE_OK(rc, db);
-			SAFE_SQLITE3_STEP2(statement1);
-			rc=(*proxy_sqlite3_clear_bindings)(statement1); ASSERT_SQLITE_OK(rc, db);
-			// Note: no assert() in proxy_debug_func() after sqlite3_reset() because it is possible that we are in shutdown
-			rc=(*proxy_sqlite3_reset)(statement1); // ASSERT_SQLITE_OK(rc, db);
+			// Create a DebugLogEntry
+			DebugLogEntry entry;
+			entry.time = curtime;
+			entry.lapse = curtime - pretime;
+			entry.thr = thr;
+			entry.file = __file;
+			entry.line = __line;
+			entry.funct = __func;
+			entry.module = module;
+			entry.modname = GloVars.global.gdbg_lvl[module].name;
+			entry.verbosity = verbosity;
+			entry.message = origdebugbuff;
+			entry.backtrace = longdebugbuff2;
+			log_buffer.push_back(entry);
+			// we now batch writes
+			// note1: in case of crash, the database will have some missing entries,
+			// but the entries can be read in `log_buffer` in the core dump
+			// note2: also in case of shutdown , `log_buffer` will have entries that won't be saved.
+			// if we really want *all* entries, we could just call sync_log_buffer_to_disk() on shutdown
+			if (
+				(log_buffer.size() >= limitSize)
+				||
+				(entry.file == "ProxySQL_Admin.cpp" && entry.funct == "flush_logs")
+			) {
+				sync_log_buffer_to_disk(db);
+			}
 		}
 	}
 	pthread_mutex_unlock(&debug_mutex);
@@ -518,4 +565,5 @@ void proxysql_set_admin_debugdb_disk(SQLite3DB * _db) {
 void proxysql_set_admin_debug_output(unsigned int _do) {
 	debug_output = _do;
 }
+
 #endif /* DEBUG */
