@@ -277,3 +277,76 @@ std::vector<std::string> split_string(const std::string& str, char delimiter) {
 
 	return tokens;
 }
+
+char* escape_string_single_quotes_and_backslashes(char* input, bool free_it) {
+	const char* c;
+	int input_len = 0;
+	int escape_count = 0;
+
+	for (c = input; *c != '\0'; c++) {
+		if ((*c == '\'') || (*c == '\\')) {
+			escape_count += 2;
+		}
+		input_len++;
+	}
+
+	if (escape_count == 0)
+		return input;
+
+	char* output = (char*)malloc(input_len + escape_count + 1);
+	char* p = output;
+
+	for (c = input; *c != '\0'; c++) {
+		if ((*c == '\'') || (*c == '\\')) {
+			*(p++) = '\\';
+		}
+		*(p++) = *c;
+	}
+	*(p++) = '\0';
+	if (free_it) {
+		free(input);
+	}
+	return output;
+}
+
+/**
+ * Escapes spaces in the input string by prepending "\\".
+ * If no spaces are present, the original input is returned.
+ * If spaces are escaped, a new string is returned, and the caller
+ * is responsible for freeing it.
+ *
+ * @param input The input string to process.
+ * @return A new string with spaces escaped, or the original input string if no escaping is needed.
+ */
+const char* escape_string_backslash_spaces(const char* input) {
+	const char* c;
+	int input_len = 0;
+	int escape_count = 0;
+
+	for (c = input; *c != '\0'; c++) {
+		if ((*c == ' ')) {
+			escape_count += 3;
+		} else if ((*c == '\\')) {
+			escape_count += 2;
+		}
+		input_len++;
+	}
+
+	if (escape_count == 0)
+		return input;
+
+	char* output = (char*)malloc(input_len + escape_count + 1);
+	char* p = output;
+
+	for (c = input; *c != '\0'; c++) {
+		if ((*c == ' ')) {
+			memcpy(p, "\\\\", 2);
+			p += 2;
+		} else if (*c == '\\') {
+			*(p++) = '\\';
+		}
+		*(p++) = *c;
+	}
+	*(p++) = '\0';
+	return output;
+}
