@@ -27,16 +27,16 @@ fi
 
 pushd percona-scheduler || exit
 
-if [[ ! -e $(command -v bash 2> /dev/null)  ]]; then
+if [[ ! -e $(command -v go 2> /dev/null)  ]]; then
   error "" "go packages not found. Please install golang package."
   exit 1
 fi
 
-/usr/bin/go/bin/go mod tidy
-/usr/bin/go/bin/go build -v -a -ldflags "-X main.pxcSchedulerHandlerVersion=${PROXYSQL_ADMIN_VERSION}" -o pxc_scheduler_handler
+go mod tidy
+go build -v -a -ldflags "-X main.pxcSchedulerHandlerVersion=${PROXYSQL_ADMIN_VERSION}" -o pxc_scheduler_handler
 
 if [ $? -ne 0 ]; then
-  error "" "/usr/bin/go/bin/go build process failed with errors. Exiting.."
+  error "" "go build process failed with errors. Exiting.."
   exit 1
 fi
 
