@@ -9,7 +9,7 @@
 #include "MySQL_Logger.hpp"
 #include "MySQL_Data_Stream.h"
 #include "proxysql_utils.h"
-#include "query_processor.h"
+#include "MySQL_Query_Processor.h"
 #include "SQLite3_Server.h"
 
 #include <search.h>
@@ -99,10 +99,10 @@ static char *s_strdup(char *s) {
 
 static int __SQLite3_Server_refresh_interval=1000;
 
-extern Query_Cache *GloQC;
+extern MySQL_Query_Cache *GloMyQC;
 extern MySQL_Authentication *GloMyAuth;
 extern ProxySQL_Admin *GloAdmin;
-extern Query_Processor *GloQPro;
+extern MySQL_Query_Processor* GloMyQPro;
 extern MySQL_Threads_Handler *GloMTH;
 extern MySQL_Logger *GloMyLogger;
 extern MySQL_Monitor *GloMyMon;
@@ -357,7 +357,7 @@ vector<aurora_hg_info_t> get_hgs_info(SQLite3DB* db) {
 
 #endif
 
-void SQLite3_Server_session_handler(MySQL_Session *sess, void *_pa, PtrSize_t *pkt) {
+void SQLite3_Server_session_handler(MySQL_Session* sess, void *_pa, PtrSize_t *pkt) {
 
 	char *error=NULL;
 	int cols;
@@ -871,7 +871,13 @@ __run_query:
 			}
 #endif // TEST_READONLY
 #ifdef TEST_REPLICATIONLAG
-			if (strncasecmp("SELECT SLAVE STATUS ", query_no_space, strlen("SELECT SLAVE STATUS ")) == 0) {
+			if (
+				strncasecmp("SELECT SLAVE STATUS ", query_no_space, strlen("SELECT SLAVE STATUS ")) == 0
+				|| strncasecmp("SELECT REPLICA STATUS ", query_no_space, strlen("SELECT REPLICA STATUS ")) == 0
+			) {
+				uint64_t addr_offset {
+					strstr(query_no_space, "REPLICA") ?  strlen("SELECT REPLICA STATUS ") : strlen("SELECT SLAVE STATUS ")
+				};
 				if (strlen(query_no_space) > strlen("SELECT SLAVE STATUS ") + 5) {
 					pthread_mutex_lock(&GloSQLite3Server->test_replicationlag_mutex);
 					// the current test doesn't try to simulate failures, therefore it will return immediately
@@ -879,17 +885,15 @@ __run_query:
 						// probably never initialized
 						GloSQLite3Server->load_replicationlag_table(sess);
 					}
-					const int* rc = GloSQLite3Server->replicationlag_test_value(query_no_space + strlen("SELECT SLAVE STATUS "));
+					const int* rc = GloSQLite3Server->replicationlag_test_value(query_no_space + addr_offset);
 					free(query);
-					if (rc == nullptr) {
-						const char* a = (char*)"SELECT null as Seconds_Behind_Master";
-						query = (char*)malloc(strlen(a) + 2);
-						sprintf(query, a);
-					} else {
-						const char* a = (char*)"SELECT %d as Seconds_Behind_Master";
-						query = (char*)malloc(strlen(a) + 2);
-						sprintf(query, a, *rc);
-					}
+
+					string SELECT { "SELECT " + (rc ? std::to_string(*rc) : string { "null" }) + " AS " };
+					SELECT += strstr(query_no_space, "REPLICA") ? "Seconds_Behind_Source" : "Seconds_Behind_Master";
+
+					query = static_cast<char*>(malloc(SELECT.size() + 1));
+					sprintf(query, SELECT.c_str());
+
 					pthread_mutex_unlock(&GloSQLite3Server->test_replicationlag_mutex);
 				}
 			}
@@ -962,7 +966,8 @@ __run_query:
 		if ((*proxy_sqlite3_get_autocommit)(db)==0) {
 			in_trans = true;
 		}
-		sess->SQLite3_to_MySQL(resultset, error, affected_rows, &sess->client_myds->myprot, in_trans);
+		bool deprecate_eof = sess->client_myds->myconn->options.client_flag & CLIENT_DEPRECATE_EOF;
+		sess->SQLite3_to_MySQL(resultset, error, affected_rows, &sess->client_myds->myprot, in_trans, deprecate_eof);
 		delete resultset;
 #ifdef TEST_READONLY
 		if (strncasecmp("SELECT",query_no_space,6)) {
@@ -1042,9 +1047,9 @@ static void *child_mysql(void *arg) {
 	SQLite3_Session *sqlite_sess = new SQLite3_Session();
 	mysql_thr->gen_args = (void *)sqlite_sess;
 
-	GloQPro->init_thread();
+	GloMyQPro->init_thread();
 	mysql_thr->refresh_variables();
-	MySQL_Session *sess=mysql_thr->create_new_session_and_client_data_stream(client);
+	MySQL_Session *sess=mysql_thr->create_new_session_and_client_data_stream<MySQL_Thread, MySQL_Session*>(client);
 	sess->thread=mysql_thr;
 	sess->session_type = PROXYSQL_SESSION_SQLITE;
 	sess->handler_function=SQLite3_Server_session_handler;
@@ -1054,7 +1059,7 @@ static void *child_mysql(void *arg) {
 	fds[0].revents=0;
 	fds[0].events=POLLIN|POLLOUT;
 	free(arg);
-	sess->client_myds->myprot.generate_pkt_initial_handshake(true,NULL,NULL, &sess->thread_session_id, false);
+	sess->client_myds->myprot.generate_pkt_initial_handshake(true,NULL,NULL, &sess->thread_session_id, true);
 
 	while (__sync_fetch_and_add(&glovars.shutdown,0)==0) {
 		if (myds->available_data_out()) {
