@@ -19,6 +19,9 @@
 */
 
 #include "httpserver/deferred_response.hpp"
+#include <microhttpd.h>
+
+struct MHD_Response;
 
 using namespace std;
 
