@@ -99,7 +99,7 @@ get_sources(){
         echo "Sources will not be downloaded"
         return 0
     fi
-    PRODUCT=proxysql2
+    PRODUCT=proxysql3
     echo "PRODUCT=${PRODUCT}" > proxysql.properties
     PRODUCT_FULL=${PRODUCT}-${VERSION}
     echo "VERSION=${VERSION}" >> proxysql.properties
@@ -129,6 +129,7 @@ get_sources(){
         git checkout ${PAT_TAG}
     fi
     sed -i 's:2.0.12:2.0.15:' proxysql-admin
+    #update_pat
     cd ..
     git clone ${PROXYSQL_REPO} ${PRODUCT_FULL}
     retval=$?
@@ -142,8 +143,7 @@ get_sources(){
     git checkout ${PROXYSQL_BRANCH}
     GIT_VERSION=${VERSION}-percona
     REVISION=$(git rev-parse --short HEAD)
-    sed -i "s|^GIT_VERSION ?= .*|GIT_VERSION=${VERSION}-percona|" Makefile
-    sed -i '0,/^endif$/s/^endif$/endif\nexport GIT_VERSION := $(GIT_VERSION)/' Makefile
+    #sed -i "s|^GIT_VERSION ?= .*|GIT_VERSION=${VERSION}-percona|" Makefile
     sed -i "s/export CURVER?=.*/export CURVER?=${VERSION}/g" Makefile
     sed -i 's/shell cat.*/shell rpm --eval \%rhel)/' deps/Makefile
     sed -i 's:6.7:6:' deps/Makefile
@@ -164,7 +164,7 @@ get_sources(){
     source proxysql.properties
     #
 
-    tar --owner=0 --group=0 --exclude=.git -czf ${PRODUCT_FULL}.tar.gz ${PRODUCT_FULL}
+    tar --owner=0 --group=0 -czf ${PRODUCT_FULL}.tar.gz ${PRODUCT_FULL}
     echo "UPLOAD=UPLOAD/experimental/BUILDS/${PRODUCT}/${PRODUCT_FULL}/${PSM_BRANCH}/${REVISION}/${BUILD_ID}" >> proxysql.properties
     mkdir $WORKDIR/source_tarball
     mkdir $CURDIR/source_tarball
@@ -192,13 +192,19 @@ get_system(){
 install_go() {
     export PATH=$PATH:/usr/bin/go/bin
     if [ x"$ARCH" = "xx86_64" ]; then
-        wget https://go.dev/dl/go1.23.12.linux-amd64.tar.gz
+        until wget https://go.dev/dl/go1.23.12.linux-amd64.tar.gz; do
+            echo "Service not ready, retrying in 10 seconds..."
+            sleep 10
+        done
         rm -rf /usr/bin/go
         tar -C /usr/bin -xzf go1.23.12.linux-amd64.tar.gz
         #update-alternatives --install /usr/bin/go go /usr/local/go/bin/go 1
         #update-alternatives --set go /usr/local/go/bin/go
     else
-        wget https://go.dev/dl/go1.23.12.linux-arm64.tar.gz
+        until wget https://go.dev/dl/go1.23.12.linux-arm64.tar.gz; do
+            echo "Service not ready, retrying in 10 seconds..."
+            sleep 10
+        done
         rm -rf /usr/bin/go
         tar -C /usr/bin -xzf go1.23.12.linux-arm64.tar.gz
         #update-alternatives --install /usr/bin/go go /usr/local/go/bin/go 1
@@ -414,6 +420,7 @@ install_deps() {
       #update_pat
       apt-get install -y libicu-dev libevent-dev
     fi
+    apt-get install -y libicu-dev libevent-dev
     return;
 }
 
@@ -479,7 +486,25 @@ build_srpm(){
     tar vxzf ${WORKDIR}/${TARFILE} --wildcards '*/rpm' --strip=1
     #
     cp -av rpm/* rpmbuild/SOURCES
+    cd ${WORKDIR}/${PRODUCT_FULL}/proxysql-admin-tool
+    update_pat
+    cd $WORKDIR
+    cp -ap ${WORKDIR}/${PRODUCT_FULL}/proxysql-admin-tool/* rpmbuild/SOURCES/
+    cp -ap ${WORKDIR}/${PRODUCT_FULL}/proxysql-admin-tool/percona-scheduler/pxc_scheduler_handler rpmbuild/SOURCES/
+    cp -ap ${WORKDIR}/${PRODUCT_FULL}/proxysql-admin-tool/config.toml rpmbuild/SOURCES/
+    cd rpmbuild/SOURCES
+    
+   # wget --no-check-certificate https://download.osgeo.org/postgis/docs/postgis-3.3.1.pdf
+    #wget --no-check-certificate https://www.postgresql.org/files/documentation/pdf/12/postgresql-12-A4.pdf
+    cd ../../
     cp -av rpmbuild/SOURCES/proxysql.spec rpmbuild/SPECS
+#    cd ${WORKDIR}/rpmbuild/SPECS
+#    cp -ap ${WORKDIR}/proxysql-packaging/rpm/*.spec .
+#    cp -ap ${WORKDIR}/proxysql-admin-tool/* rpmbuild/SOURCES/
+#    cp -ap ${WORKDIR}/proxysql-admin-tool/percona-scheduler/pxc_scheduler_handler rpmbuild/SOURCES/
+#    cp -ap ${WORKDIR}/proxysql-admin-tool/config.toml rpmbuild/SOURCES/
+    cd ${WORKDIR}
+    #
     mv -fv ${TARFILE} ${WORKDIR}/rpmbuild/SOURCES
    # if [ -f /opt/rh/devtoolset-7/enable ]; then
    #     source /opt/rh/devtoolset-7/enable
@@ -528,18 +553,9 @@ build_rpm(){
     mkdir -vp rpmbuild/{SOURCES,SPECS,BUILD,SRPMS,RPMS}
     cp $SRC_RPM rpmbuild/SRPMS/
 
-    git clone ${PAT_REPO}
-    cd proxysql-admin-tool
-    git fetch origin
-    if [ ! -z ${PAT_TAG} ]; then
-        git checkout ${PAT_TAG}
-    fi
-    sed -i 's:2.0.12:2.0.15:' proxysql-admin
-    update_pat
-    cd ${WORKDIR}
-    cp -ap ${WORKDIR}/proxysql-admin-tool/* rpmbuild/SOURCES/
-    cp -ap ${WORKDIR}/proxysql-admin-tool/percona-scheduler/pxc_scheduler_handler rpmbuild/SOURCES/
-    cp -ap ${WORKDIR}/proxysql-admin-tool/config.toml rpmbuild/SOURCES/
+    cd rpmbuild/SRPMS/
+    #
+    cd $WORKDIR
     RHEL=$(rpm --eval %rhel)
     ARCH=$(echo $(uname -m) | sed -e 's:i686:i386:g')
     [ -f /opt/percona-devtoolset/enable ] && source /opt/percona-devtoolset/enable
@@ -566,7 +582,7 @@ build_source_deb(){
         echo "It is not possible to build source deb here"
         exit 1
     fi
-    #rm -rf proxysql2*
+    #rm -rf proxysql3*
     get_tar "source_tarball"
     rm -f *.dsc *.orig.tar.gz *.debian.tar.gz *.changes
     #
@@ -578,7 +594,15 @@ build_source_deb(){
     #
     
     mv ${TARFILE} ${PRODUCT}_${VERSION}.orig.tar.gz
-    cd ${BUILDDIR}/debian
+    cd ${BUILDDIR}/proxysql-admin-tool
+    #update_pat
+    cd ..
+    cp -ap proxysql-admin-tool/* tools/
+    cp -ap proxysql-admin-tool/percona-scheduler/pxc_scheduler_handler tools/
+    cp -ap proxysql-admin-tool/config.toml etc/
+    mv tools/LICENSE .
+    mv tools/proxysql-admin.cnf etc/ 
+    cd debian
     #rm -rf changelog
     sed -i "s:@@VERSION@@:${VERSION}:g" changelog
     sed -i "s:@@VERSION@@:${VERSION}:g" control
@@ -626,8 +650,8 @@ build_tarball(){
     git checkout ${GIT_BRANCH}
     cd $WORKDIR
     gcc --version
-    sed -i 's/$SOURCEDIR\/Makefile/$SOURCEDIR\/proxysql2-${PROXYSQL_VERSION}\/Makefile/' ./proxysql-packaging/scripts/build_binary.sh
-    sed -i 's/cd $SOURCEDIR/cd $SOURCEDIR\/proxysql2-${PROXYSQL_VERSION}/' ./proxysql-packaging/scripts/build_binary.sh
+    sed -i 's/$SOURCEDIR\/Makefile/$SOURCEDIR\/proxysql3-${PROXYSQL_VERSION}\/Makefile/' ./proxysql-packaging/scripts/build_binary.sh
+    sed -i 's/cd $SOURCEDIR/cd $SOURCEDIR\/proxysql3-${PROXYSQL_VERSION}/' ./proxysql-packaging/scripts/build_binary.sh
     sed -i '73i source $SOURCEDIR/proxysql.properties' ./proxysql-packaging/scripts/build_binary.sh
     sed -i '248i cp proxysql-$VERSION-$(uname -s)-$(uname -m)$GLIBC_VER.tar.gz $SOURCEDIR' ./proxysql-packaging/scripts/build_binary.sh
     sed -i 's/s|go build|${BINGO} build|g/s|go build|\/usr\/bin\/go\/bin\/go build|g/' ./proxysql-packaging/scripts/build_binary.sh
@@ -690,19 +714,13 @@ build_deb(){
     #
     dpkg-source -x ${DSC}
     #
-    git clone ${PAT_REPO}
-    cd proxysql-admin-tool
-    git fetch origin
-    if [ ! -z ${PAT_TAG} ]; then
-        git checkout ${PAT_TAG}
-    fi
-    sed -i 's:2.0.12:2.0.15:' proxysql-admin
-    update_pat
-    cd ${WORKDIR}
     cd ${PRODUCT}-${VERSION}
-    cp -ap ${WORKDIR}/proxysql-admin-tool/* tools/
-    cp -ap ${WORKDIR}/proxysql-admin-tool/percona-scheduler/pxc_scheduler_handler tools/
-    cp -ap ${WORKDIR}/proxysql-admin-tool/config.toml etc/
+    cd proxysql-admin-tool
+    update_pat
+    cd ..
+    cp -ap proxysql-admin-tool/* tools/
+    cp -ap proxysql-admin-tool/percona-scheduler/pxc_scheduler_handler tools/
+    cp -ap proxysql-admin-tool/config.toml etc/
     mv tools/LICENSE .
     mv tools/proxysql-admin.cnf etc/
     sed -i "s:@@VERSION@@:${VERSION}:g" debian/changelog
@@ -763,7 +781,7 @@ PAT_REPO=https://github.com/percona/proxysql-admin-tool.git
 PAT_TAG="v2.5.3-dev"
 PROXYSQL_BRANCH="v2.5.3"
 PROXYSQL_REPO="https://github.com/sysown/proxysql.git"
-PRODUCT=proxysql2
+PRODUCT=proxysql3
 DEBUG=0
 parse_arguments PICK-ARGS-FROM-ARGV "$@"
 VERSION=${VERSION}
