@@ -23,6 +23,20 @@ tokenizer_t;
 
 enum { TOKENIZER_EMPTIES_OK, TOKENIZER_NO_EMPTIES };
 
+/**
+ * @brief Struct for holding all the configuration options used for query digests generation.
+ */
+typedef struct _options {
+	bool lowercase;
+	bool replace_null;
+	bool replace_number;
+	bool keep_comment;
+	int grouping_limit;
+	int groups_grouping_limit;
+	int max_query_length;
+} options;
+
+
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
@@ -34,8 +48,9 @@ char * mysql_query_digest_first_stage(const char* const q, int q_len, char** con
 char * mysql_query_digest_second_stage(const char* const q, int q_len, char** const fst_cmnt, char* const buf);
 char * mysql_query_digest_and_first_comment_2(const char* const q, int q_len, char** const fst_cmnt, char* const buf);
 char * mysql_query_digest_and_first_comment_one_it(char *s , int len , char **first_comment, char *buf);
-char * mysql_query_strip_comments(char *s , int len);
 void c_split_2(const char *in, const char *del, char **out1, char **out2);
+char * query_strip_comments(char* s, int len, bool lowercase);
+char * query_digest_and_first_comment_2(const char* const q, int q_len, char** const fst_cmnt, char* const buf, const options* opts);
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
