@@ -29,6 +29,8 @@ using std::vector;
 using std::to_string;
 using nlohmann::json;
 
+#define SELECT_RUNTIME_VAR "SELECT variable_value FROM runtime_global_variables WHERE variable_name="
+
 #define LAST_QUERY_EXECUTED_STR(mysql)	(*static_cast<std::string*>(mysql->unused_0)) 
 #define STMT_VECTOR(stmt)				(*static_cast<std::vector<MYSQL_STMT*>*>(stmt->mysql->unused_3))
 #define STMT_EXECUTED_VECTOR(stmt)		(*static_cast<std::vector<std::unique_ptr<char,decltype(&free)>>*>(stmt->mysql->unused_4))
@@ -187,6 +189,76 @@ my_bool mysql_stmt_close_override(MYSQL_STMT* stmt, const char* file, int line)
 
 #endif
 
+long long binom_coeff(int n, int k) {
+    if (k > n) return 0;
+    if (k == 0 || k == n) return 1;
+
+    long long res = 1;
+
+    for (int i = 0; i < k; ++i) {
+        res *= (n - i);
+        res /= (i + 1);
+    }
+
+    return res;
+}
+
+long double prob_filled(int N, int M) {
+    if (N < M) return 0.0;
+
+    double prob_empty = 0.0;
+
+    for (int k = 1; k <= M; ++k) {
+        double binom = binom_coeff(M, k);
+        double base = (double(M) - k)/ double(M);
+        double term = binom * pow(base, double(N));
+
+        prob_empty += (k % 2 == 1 ? 1 : -1) * term;
+    }
+
+    return 1.0 - prob_empty;
+}
+
+int find_min_elems(double tg_prob, int M) {
+    int low = M, high = 20 * M, mid;
+
+    while (low < high) {
+        mid = (low + high) / 2;
+        double prob = prob_filled(mid, M);
+
+        if (prob < tg_prob) {
+            low = mid + 1;
+        } else {
+            high = mid;
+        }
+    }
+
+    return low;
+}
+
+string to_string(std::thread::id id) {
+	std::stringstream helper;
+	helper << id;
+
+	return helper.str();
+}
+
+string replace_str(const string& str, const string& match, const string& repl) {
+	if(match.empty()) {
+		return str;
+	}
+
+	string result { str };
+	size_t start_pos = 0;
+
+	while((start_pos = result.find(match, start_pos)) != std::string::npos) {
+		result.replace(start_pos, match.length(), repl);
+		start_pos += repl.length();
+	}
+
+	return result;
+}
+
 pair<int,vector<MYSQL*>> disable_core_nodes_scheduler(CommandLine& cl, MYSQL* admin) {
 	vector<MYSQL*> nodes_conns {};
 
@@ -239,7 +311,7 @@ std::size_t count_matches(const string& str, const string& substr) {
 }
 
 int mysql_query_t__(MYSQL* mysql, const char* query, const char* f, int ln, const char* fn) {
-	diag("%s:%d:%s(): Issuing query '%s' to ('%s':%d)", f, ln, fn, query, mysql->host, mysql->port);
+	diag("%s:%d:%s(): Issuing query \"%s\" to ('%s':%d)", f, ln, fn, query, mysql->host, mysql->port);
 	return mysql_query(mysql, query);
 }
 
@@ -555,15 +627,15 @@ std::vector<mysql_res_row> extract_mysql_rows(MYSQL_RES* my_res) {
 	return result;
 };
 
-pair<uint32_t,vector<mysql_res_row>> mysql_query_ext_rows(MYSQL* mysql, const string& query) {
+rc_t<vector<mysql_res_row>> mysql_query_ext_rows(MYSQL* mysql, const string& query) {
 	int rc = mysql_query(mysql, query.c_str());
 	if (rc != EXIT_SUCCESS) {
-		return { mysql_errno(mysql), {} };
+		return { static_cast<int>(mysql_errno(mysql)), {} };
 	}
 
 	MYSQL_RES* myres = mysql_store_result(mysql);
 	if (myres == nullptr) {
-		return { mysql_errno(mysql), {} };
+		return { static_cast<int>(mysql_errno(mysql)), {} };
 	}
 
 	const vector<mysql_res_row> rows { extract_mysql_rows(myres) };
@@ -584,9 +656,9 @@ ext_val_t<int32_t> ext_single_row_val(const mysql_res_row& row, const int32_t& d
 	if (row.empty() || row.front().empty()) {
 		return { -1, def_val, {} };
 	} else {
-        errno = 0;
-        char* p_end {};
-        const int32_t val = std::strtol(row.front().c_str(), &p_end, 10);
+		errno = 0;
+		char* p_end {};
+		const int32_t val = std::strtol(row.front().c_str(), &p_end, 10);
 
 		if (row[0] == p_end || errno == ERANGE) {
 			return { -2, def_val, string { row[0] } };
@@ -600,9 +672,9 @@ ext_val_t<uint32_t> ext_single_row_val(const mysql_res_row& row, const uint32_t&
 	if (row.empty() || row.front().empty()) {
 		return { -1, def_val, {} };
 	} else {
-        errno = 0;
-        char* p_end {};
-        const uint32_t val = std::strtoul(row.front().c_str(), &p_end, 10);
+		errno = 0;
+		char* p_end {};
+		const uint32_t val = std::strtoul(row.front().c_str(), &p_end, 10);
 
 		if (row[0] == p_end || errno == ERANGE) {
 			return { -2, def_val, string { row[0] } };
@@ -617,9 +689,9 @@ ext_val_t<int64_t> ext_single_row_val(const mysql_res_row& row, const int64_t& d
 	if (row.empty() || row.front().empty()) {
 		return { -1, def_val, {} };
 	} else {
-        errno = 0;
-        char* p_end {};
-        const int64_t val = std::strtoll(row.front().c_str(), &p_end, 10);
+		errno = 0;
+		char* p_end {};
+		const int64_t val = std::strtoll(row.front().c_str(), &p_end, 10);
 
 		if (row[0] == p_end || errno == ERANGE) {
 			return { -2, def_val, string { row[0] } };
@@ -633,9 +705,9 @@ ext_val_t<uint64_t> ext_single_row_val(const mysql_res_row& row, const uint64_t&
 	if (row.empty() || row.front().empty()) {
 		return { -1, def_val, {} };
 	} else {
-        errno = 0;
-        char* p_end {};
-        const uint64_t val = std::strtoull(row.front().c_str(), &p_end, 10);
+		errno = 0;
+		char* p_end {};
+		const uint64_t val = std::strtoull(row.front().c_str(), &p_end, 10);
 
 		if (row[0] == p_end || errno == ERANGE) {
 			return { -2, def_val, string { row[0] } };
@@ -981,29 +1053,30 @@ int create_extra_users(
 	return EXIT_SUCCESS;
 }
 
-string tap_curtime() {
-	time_t __timer;
-	char lut[30];
-	struct tm __tm_info;
-	time(&__timer);
-	localtime_r(&__timer, &__tm_info);
-	strftime(lut, 25, "%Y-%m-%d %H:%M:%S", &__tm_info);
-	string s = string(lut);
-	return s;
-}
-
-int get_proxysql_cpu_usage(const CommandLine& cl, uint32_t intv, double& cpu_usage) {
+int get_proxysql_cpu_usage(const CommandLine& cl, double& cpu_usage, uint32_t intv) {
 	// Create Admin connection
-	MYSQL* proxysql_admin = mysql_init(NULL);
-	if (!mysql_real_connect(proxysql_admin, cl.host, cl.admin_username, cl.admin_password, NULL, cl.admin_port, NULL, 0)) {
-		fprintf(stderr, "File %s, line %d, Error: %s\n", __FILE__, __LINE__, mysql_error(proxysql_admin));
+	MYSQL* admin = mysql_init(NULL);
+	if (!mysql_real_connect(admin, cl.host, cl.admin_username, cl.admin_password, NULL, cl.admin_port, NULL, 0)) {
+		fprintf(stderr, "File %s, line %d, Error: %s\n", __FILE__, __LINE__, mysql_error(admin));
 		return EXIT_FAILURE;
 	}
 
-	// Set new interval
-	const string set_stats_query { "SET admin-stats_system_cpu=" + std::to_string(intv) };
-	MYSQL_QUERY(proxysql_admin, set_stats_query.c_str());
-	MYSQL_QUERY(proxysql_admin, "LOAD ADMIN VARIABLES TO RUNTIME");
+	const char SELECT_CPU_INTV[] { SELECT_RUNTIME_VAR"'admin-stats_system_cpu'" };
+	ext_val_t<int32_t> cur_intv { mysql_query_ext_val(admin, SELECT_CPU_INTV, -1) };
+
+	if (cur_intv.err) {
+		const string err { get_ext_val_err(admin, cur_intv) };
+		diag("Failed query   query:`%s`, err:`%s`", SELECT_CPU_INTV, err.c_str());
+		return EXIT_FAILURE;
+	}
+
+	if (intv == UINT32_MAX) {
+		intv = cur_intv.val;
+	} else {
+		const string set_stats_query { "SET admin-stats_system_cpu=" + std::to_string(intv) };
+		MYSQL_QUERY(admin, set_stats_query.c_str());
+		MYSQL_QUERY(admin, "LOAD ADMIN VARIABLES TO RUNTIME");
+	}
 
 	// Wait until 'system_cpu' is filled with newer entries
 	time_t curtime = time(NULL);
@@ -1012,9 +1085,9 @@ int get_proxysql_cpu_usage(const CommandLine& cl, uint32_t intv, double& cpu_usa
 	const char runtime_stats[] {
 		"SELECT variable_value FROM runtime_global_variables WHERE variable_name='admin-stats_system_cpu'"
 	};
-	ext_val_t<int> ext_rintv { mysql_query_ext_val(proxysql_admin, runtime_stats, 10) };
+	ext_val_t<int> ext_rintv { mysql_query_ext_val(admin, runtime_stats, 10) };
 	if (ext_rintv.err != EXIT_SUCCESS) {
-		const string err { get_ext_val_err(proxysql_admin, ext_rintv) };
+		const string err { get_ext_val_err(admin, ext_rintv) };
 		diag("Failed query   query:`%s`, err:`%s`", runtime_stats, err.c_str());
 		return EXIT_FAILURE;
 	}
@@ -1034,10 +1107,10 @@ int get_proxysql_cpu_usage(const CommandLine& cl, uint32_t intv, double& cpu_usa
 	const string count_query {
 		"SELECT COUNT(*) FROM system_cpu WHERE timestamp > " + std::to_string(curtime)
 	};
-	ext_val_t<int> ext_stats_count { mysql_query_ext_val(proxysql_admin, count_query, 10) };
+	ext_val_t<int> ext_stats_count { mysql_query_ext_val(admin, count_query, 10) };
 
 	if (ext_stats_count.err != EXIT_SUCCESS) {
-		const string err { get_ext_val_err(proxysql_admin, ext_stats_count) };
+		const string err { get_ext_val_err(admin, ext_stats_count) };
 		diag("Failed query   query:`%s`, err:`%s`", count_query.c_str(), err.c_str());
 		return EXIT_FAILURE;
 	}
@@ -1047,10 +1120,10 @@ int get_proxysql_cpu_usage(const CommandLine& cl, uint32_t intv, double& cpu_usa
 
 	while (entry_count < 2) {
 		diag("Waiting for more 'system_cpu' entries...   entry_count=%d", entry_count);
-		ext_val_t<int> ext_stats_count { mysql_query_ext_val(proxysql_admin, count_query, 10) };
+		ext_val_t<int> ext_stats_count { mysql_query_ext_val(admin, count_query, 10) };
 
 		if (ext_stats_count.err != EXIT_SUCCESS) {
-			const string err { get_ext_val_err(proxysql_admin, ext_stats_count) };
+			const string err { get_ext_val_err(admin, ext_stats_count) };
 			diag("Failed query   query:`%s`, err:`%s`", count_query.c_str(), err.c_str());
 			return EXIT_FAILURE;
 		}
@@ -1059,8 +1132,11 @@ int get_proxysql_cpu_usage(const CommandLine& cl, uint32_t intv, double& cpu_usa
 		sleep(1);
 	}
 
-	MYSQL_QUERY(proxysql_admin, "SELECT * FROM system_cpu ORDER BY timestamp DESC LIMIT 2");
-	MYSQL_RES* admin_res = mysql_store_result(proxysql_admin);
+	MYSQL_QUERY(admin, "SELECT * FROM system_cpu ORDER BY timestamp DESC LIMIT 5");
+	MYSQL_RES* admin_res { mysql_store_result(admin) };
+	const string sys_cpu_str { dump_as_table(admin_res) };
+	diag("Dumping latest 5 entries from 'system_cpu':\n%s", sys_cpu_str.c_str());
+
 	MYSQL_ROW row = mysql_fetch_row(admin_res);
 
 	double s_clk = (1000.0 / sysconf(_SC_CLK_TCK));
@@ -1081,10 +1157,12 @@ int get_proxysql_cpu_usage(const CommandLine& cl, uint32_t intv, double& cpu_usa
 	mysql_free_result(admin_res);
 
 	// recover admin variables
-	MYSQL_QUERY(proxysql_admin, "SET admin-stats_system_cpu=60");
-	MYSQL_QUERY(proxysql_admin, "LOAD ADMIN VARIABLES TO RUNTIME");
+	if (intv != UINT32_MAX) {
+		MYSQL_QUERY(admin, ("SET admin-stats_system_cpu=" + std::to_string(cur_intv.val)).c_str());
+		MYSQL_QUERY(admin, "LOAD ADMIN VARIABLES TO RUNTIME");
+	}
 
-	mysql_close(proxysql_admin);
+	mysql_close(admin);
 
 	return EXIT_SUCCESS;
 }
@@ -1477,13 +1555,47 @@ int open_file_and_seek_end(const string& f_path, std::fstream& f_stream) {
 	return EXIT_SUCCESS;
 }
 
-vector<line_match_t> get_matching_lines(fstream& f_stream, const string& s_regex, bool get_matches) {
+rc_t<debug_entry_t> build_debug_entry(const sq3_row_t& row) {
+	if (row.size() < 12) {
+		return { -1, {} };
+	}
+
+	const auto str_to_uint64 = [](const std::string& str) -> uint64_t {
+		return str.empty() ? 0 : std::stoull(str);
+	};
+
+	const auto str_to_time_t = [](const string& str) -> time_t {
+		return str.empty() ? 0 : static_cast<time_t>(std::stoll(str));
+	};
+
+	return { 0, debug_entry_t {
+		str_to_uint64(row[0]),
+		str_to_time_t(row[1]),
+		str_to_uint64(row[2]),
+		str_to_uint64(row[3]),
+		row[4],
+		str_to_uint64(row[5]),
+		row[6],
+		str_to_uint64(row[7]),
+		row[8],
+		str_to_uint64(row[9]),
+		row[10],
+		row[11]
+	}};
+}
+
+pair<size_t,vector<line_match_t>> get_matching_lines(
+	fstream& f_stream, const string& s_regex, bool get_matches
+) {
 	vector<line_match_t> found_matches {};
+	size_t insp_lines { 0 };
 
 	string next_line {};
 	fstream::pos_type init_pos { f_stream.tellg() };
 
 	while (getline(f_stream, next_line)) {
+		insp_lines += 1;
+
 		re2::RE2 regex { s_regex };
 		re2::StringPiece match;
 
@@ -1506,7 +1618,7 @@ vector<line_match_t> get_matching_lines(fstream& f_stream, const string& s_regex
 		f_stream.seekg(init_pos);
 	}
 
-	return found_matches;
+	return { insp_lines, found_matches };
 }
 
 const uint32_t USLEEP_SQLITE_LOCKED = 100;
@@ -1538,7 +1650,7 @@ sq3_res_t sqlite3_execute_stmt(sqlite3* db, const string& query) {
 	} while (rc==SQLITE_LOCKED || rc==SQLITE_BUSY);
 
 	if (rc != SQLITE_OK) {
-		res = {{}, {}, {}, sqlite3_errmsg(db)};
+		res = {{}, {}, {}, sqlite3_errcode(db)};
 		goto cleanup;
 	}
 
@@ -1558,7 +1670,7 @@ sq3_res_t sqlite3_execute_stmt(sqlite3* db, const string& query) {
 				uint32_t affected_rows = sqlite3_changes(db);
 				res = {{}, {}, affected_rows, {}};
 			} else {
-				res = {{}, {}, {}, sqlite3_errmsg(db)};
+				res = {{}, {}, {}, sqlite3_errcode(db)};
 				goto cleanup;
 			}
 		} else {
@@ -1569,7 +1681,12 @@ sq3_res_t sqlite3_execute_stmt(sqlite3* db, const string& query) {
 				cols_defs.push_back(sqlite3_column_name(stmt, i));
 			}
 
-			while ((rc = sqlite3_step(stmt)) == SQLITE_ROW) {
+			while ((rc = sqlite3_step(stmt)) == SQLITE_ROW || rc == SQLITE_BUSY) {
+				if (rc == SQLITE_BUSY) {
+					usleep(USLEEP_SQLITE_LOCKED);
+					continue;
+				}
+
 				sq3_row_t row {};
 
 				for (uint32_t i = 0; i < cols_count; i++) {
@@ -1594,6 +1711,31 @@ cleanup:
 	return res;
 }
 
+rc_t<vector<debug_entry_t>> sq3_get_debug_entries(sqlite3* db, const string& conds) {
+	const string query {
+		"SELECT id, time, lapse, thread, file, line, funct, modnum, modname, verbosity, message, note"
+		" FROM debug_log" + (conds.empty() ? "" : " WHERE 1=1 AND " + conds)
+	};
+
+	const sq3_res_t rows { sqlite3_execute_stmt(db, query) };
+	const int sq3_err { std::get<SQ3_RES_T::SQ3_ERR>(rows) };
+
+	if (sq3_err) {
+		return { sq3_err, {} };
+	} {
+		vector<debug_entry_t> entries {};
+
+		for (const auto & row : std::get<SQ3_RES_T::SQ3_ROWS>(rows)) {
+			auto [err, entry] = build_debug_entry(row);
+			assert((void("Received malformed row from 'debug_log' table"), err == 0));
+
+			entries.push_back(entry);
+		}
+
+		return { 0, entries };
+	}
+}
+
 json fetch_internal_session(MYSQL* proxy, bool verbose) {
 	int rc = 0;
 
@@ -1791,6 +1933,61 @@ int dump_conn_stats(MYSQL* admin, const vector<uint32_t> hgs) {
 	return EXIT_SUCCESS;
 }
 
+string row_to_str(const mysql_res_row& row) {
+	string res { "[" };
+
+	for (const auto& e : row) {
+		res += "\"" + e + "\"";
+
+		if (&e != &row.back()) {
+			res += ",";
+		}
+	}
+
+	res += "]";
+
+	return res;
+}
+
+ext_val_t<hg_pool_st_t> ext_single_row_val(const mysql_res_row& row, const hg_pool_st_t& def_val) {
+	if (row.empty() || row.size() != sizeof(hg_pool_st_t)/sizeof(uint32_t)) {
+		return { -1, def_val, {} };
+	} else {
+		for (int i = 0; i < sizeof(hg_pool_st_t)/sizeof(uint32_t); i++) {
+			if (row[i].empty()) {
+				return { -1, def_val, {} };
+			}
+		}
+
+		errno = 0;
+		char* p_end { nullptr };
+		hg_pool_st_t res {};
+		const string row_str { row_to_str(row) };
+
+		res.hostgroup = std::strtoull(row.front().c_str(), &p_end, 10);
+		if (row[0].c_str() == p_end || errno == ERANGE) { return { -2, def_val, row_str }; }
+		res.conn_used = std::strtoull(row[1].c_str(), &p_end, 10);
+		if (row[1].c_str() == p_end || errno == ERANGE) { return { -2, def_val, row_str }; }
+		res.conn_free = std::strtoull(row[2].c_str(), &p_end, 10);
+		if (row[2].c_str() == p_end || errno == ERANGE) { return { -2, def_val, row_str }; }
+		res.conn_ok = std::strtoull(row[3].c_str(), &p_end, 10);
+		if (row[3].c_str() == p_end || errno == ERANGE) { return { -2, def_val, row_str }; }
+		res.conn_err = std::strtoull(row[4].c_str(), &p_end, 10);
+		if (row[4].c_str() == p_end || errno == ERANGE) { return { -2, def_val, row_str }; }
+		res.max_conn_used = std::strtoull(row[5].c_str(), &p_end, 10);
+		if (row[5].c_str() == p_end || errno == ERANGE) { return { -2, def_val, row_str }; }
+		res.max_conn_used = std::strtoull(row[6].c_str(), &p_end, 10);
+		if (row[6].c_str() == p_end || errno == ERANGE) { return { -2, def_val, row_str }; }
+
+		return { EXIT_SUCCESS, res, row_str };
+	}
+}
+
+ext_val_t<hg_pool_st_t> get_conn_pool_hg_stats(MYSQL* admin, uint32_t hg) {
+	const string HG_STATS_QUERY { gen_conn_stats_query({ hg }) };
+	return mysql_query_ext_val(admin, HG_STATS_QUERY, hg_pool_st_t {});
+}
+
 pair<int,pool_state_t> fetch_conn_stats(MYSQL* admin, const vector<uint32_t> hgs) {
 	const string stats_query { gen_conn_stats_query(hgs) };
 	const pair<int,vector<mysql_row_t>> conn_pool_stats { exec_dql_query(admin, stats_query, true) };
@@ -1843,6 +2040,8 @@ int check_cond(MYSQL* mysql, const string& q) {
 					res = 0;
 				}
 			}
+
+			mysql_free_result(myres);
 		}
 	} else {
 		diag("Check failed with error '%s'", mysql_error(mysql));
@@ -1876,7 +2075,7 @@ int wait_for_cond(MYSQL* mysql, const string& q, uint32_t to) {
 	return result;
 }
 
-vector<check_res_t> wait_for_conds(MYSQL* mysql, const vector<string>& qs, uint32_t to) {
+vector<rc_t<string>> wait_for_conds(MYSQL* mysql, const vector<string>& qs, uint32_t to) {
 	diag("Waiting multiple conditions in ('%s':%d):", mysql->host, mysql->port);
 	for (const string& q : qs) {
 		diag("  - cond: '%s'", q.c_str());
@@ -1884,10 +2083,10 @@ vector<check_res_t> wait_for_conds(MYSQL* mysql, const vector<string>& qs, uint3
 
 	std::chrono::duration<double> elapsed {};
 
-	vector<check_res_t> res {};
+	vector<rc_t<string>> res {};
 	std::transform(qs.begin(), qs.end(), std::back_inserter(res),
 		[] (const string& q) {
-			return check_res_t { 1, q };
+			return rc_t<string> { 1, q };
 		}
 	);
 	auto start = std::chrono::system_clock::now();
@@ -1908,7 +2107,7 @@ vector<check_res_t> wait_for_conds(MYSQL* mysql, const vector<string>& qs, uint3
 		}
 
 		int acc = std::accumulate(res.begin(), res.end(), size_t(0),
-			[] (size_t acc, const check_res_t& p) -> size_t {
+			[] (size_t acc, const rc_t<string>& p) -> size_t {
 				if (p.first == 0) {
 					return acc + 1;
 				} else {
@@ -1928,10 +2127,10 @@ vector<check_res_t> wait_for_conds(MYSQL* mysql, const vector<string>& qs, uint3
 	return res;
 }
 
-int proc_wait_checks(const vector<check_res_t>& chks) {
+int proc_wait_checks(const vector<rc_t<string>>& chks) {
 	int res = 0;
 
-	for (const check_res_t& r : chks) {
+	for (const rc_t<string>& r : chks) {
 		if (r.first == -1) {
 			res = -1;
 			diag("Waiting check FAILED to execute '%s'", r.second.c_str());
@@ -2058,7 +2257,7 @@ int check_nodes_sync(
 			return EXIT_FAILURE;
 		}
 
-		const vector<check_res_t> wres { wait_for_conds(admin, { check }, to) };
+		const vector<rc_t<string>> wres { wait_for_conds(admin, { check }, to) };
 		int node_sync = proc_wait_checks(wres);
 
 		if (node_sync != EXIT_SUCCESS) {
