@@ -2,18 +2,28 @@
 #define UTILS_H
 
 #include <algorithm>
+#include <functional>
 #include <string>
+#include <thread>
 #include <vector>
 #include <fstream>
+#include <unistd.h>
+#include <utility>
 
 #include "curl/curl.h"
-
+#include "mysql.h"
 #include "sqlite3db.h"
 #include "json_fwd.hpp"
 
 #include "command_line.h"
 #include "mysql.h"
 
+template <typename T>
+using rc_t = std::pair<int,T>;
+
+#define _S(s) ( std::string {s} )
+#define _TO_S(s) ( std::to_string(s) )
+
 // Improve dependency failure compilation error
 #ifndef DISABLE_WARNING_COUNT_LOGGING
 
@@ -54,8 +64,63 @@ my_bool mysql_stmt_close_override(MYSQL_STMT* stmt, const char* file, int line);
 
 }
 
+static inline int mysql_query_override(MYSQL* mysql, const std::string& query, const char* file, int line) {
+	return mysql_query_override(mysql, query.c_str(), file, line);
+}
+
+#else
+static inline int mysql_query(MYSQL* mysql, const std::string& query) {
+	return mysql_query(mysql, query.c_str());
+}
 #endif 
 
+/**
+ * @brief Simple macro to use with 'mysql_query' versions.
+ * @details E.g: `mysql_query_ext_val(admin, SELECT_RUNTIME_VAR"'mysql-eventslog_filename'", "")`.
+ */
+#define SELECT_RUNTIME_VAR "SELECT variable_value FROM runtime_global_variables WHERE variable_name="
+
+/**
+ * @brief Computes the binomial coefficient C(n, k)
+ */
+long long binom_coeff(int n, int k);
+
+/**
+ * @brief Computes the probability for all buckets has at least one elem.
+ * @param N Number of elements to place.
+ * @param M Number of buckets to fill.
+ * @return The computed probability.
+ */
+long double prob_filled(int N, int M);
+
+/**
+ * @brief Computes the inverse of 'prob_filled', finding the min number of elements so the target
+ *  probability is reached.
+ * @details NOTE: The inverse of 'prob_filled' isn't trivial to compute, due to this, this method
+ *  computes it numerically using a simple binary search. The search isn't optimized or built to work
+ *  with a generic M, yet it reasonably fits its current use cases for small M values.
+ * @param tg_prob The target probability for which to find the number of elements.
+ * @param M The number of buckets in which to place the elements.
+ * @return Number of elements necessary to achieve the target probability.
+ */
+int find_min_elems(double tg_prob, int M);
+
+/**
+ * @brief Returns the string representation of an std::thread
+ * @param id Thread id to be converted to string.
+ * @return The string representation of the thread::id.
+ */
+std::string to_string(std::thread::id id);
+
+/**
+ * @brief Replaces all occurrences of a substring in a given string with another substring.
+ * @param str The string which matches are to be replaced.
+ * @param match The string which occurrences shall be replaced.
+ * @param repl The string used to replace 'match' occurrences.
+ * @return A new string with all 'match' occurrences replaced by 'repl'.
+ */
+std::string replace_str(const std::string& str, const std::string& match, const std::string& repl);
+
 /**
  * @brief Helper function to disable Core nodes scheduler from ProxySQL Cluster nodes.
  * @details In the CI environment, 'Scheduler' is used to induce extra load via Admin interface on
@@ -68,24 +133,15 @@ my_bool mysql_stmt_close_override(MYSQL_STMT* stmt, const char* file, int line);
  */
 std::pair<int,std::vector<MYSQL*>> disable_core_nodes_scheduler(CommandLine& cl, MYSQL* admin);
 
-inline std::string get_formatted_time() {
-	time_t __timer;
-	char __buffer[30];
-
-	struct tm __tm_info {};
-	time(&__timer);
-	localtime_r(&__timer, &__tm_info);
-	strftime(__buffer, 25, "%Y-%m-%d %H:%M:%S", &__tm_info);
-
-	return std::string(__buffer);
-}
-
 /**
  * @brief Wrapper for 'mysql_query' with logging for convenience.
  * @details Should be used through 'mysql_query_t' macro.
  * @return Result of calling 'mysql_query'.
  */
 int mysql_query_t__(MYSQL* mysql, const char* query, const char* f, int ln, const char* fn);
+inline static int mysql_query_t__(MYSQL* mysql, const std::string& query, const char* f, int ln, const char* fn) {
+	return mysql_query_t__(mysql, query.c_str(), f, ln, fn);
+}
 
 /**
  * @brief Convenience macro with query logging.
@@ -112,8 +168,7 @@ int mysql_query_t__(MYSQL* mysql, const char* query, const char* f, int ln, cons
 
 #define MYSQL_QUERY_T(mysql, query) \
 	do { \
-		diag("Issuing query '%s' to ('%s':%d)", query, mysql->host, mysql->port); \
-		if (mysql_query(mysql, query)) { \
+		if (mysql_query_t(mysql, query)) { \
 			fprintf(stderr, "File %s, line %d, Error: %s\n", __FILE__, __LINE__, mysql_error(mysql)); \
 			return EXIT_FAILURE; \
 		} \
@@ -144,6 +199,10 @@ int create_table_test_sbtest1(int num_rows, MYSQL *mysql);
 int create_table_test_sqlite_sbtest1(int num_rows, MYSQL *mysql); // as above, but for SQLite3 server
 int add_more_rows_test_sbtest1(int num_rows, MYSQL *mysql, bool sqlite=false);
 
+/**
+ * @brief Returns the current monotonic time by 'clock_gettime'.
+ * @return Current monotonic time in microseconds (us).
+ */
 unsigned long long monotonic_time();
 
 using mysql_res_row = std::vector<std::string>;
@@ -164,7 +223,44 @@ std::vector<mysql_res_row> extract_mysql_rows(MYSQL_RES* my_res);
  *   * Query produces no resulset.
  *   * An error takes place during query execution or resultset retrieval.
  */
-std::pair<uint32_t,std::vector<mysql_res_row>> mysql_query_ext_rows(MYSQL* mysql, const std::string& query);
+rc_t<std::vector<mysql_res_row>> mysql_query_ext_rows(MYSQL* mysql, const std::string& query);
+
+using sq3_col_def_t = std::string;
+using sq3_row_t = std::vector<std::string>;
+using sq3_err_t = int;
+using sq3_res_t = std::tuple<std::vector<sq3_col_def_t>,std::vector<sq3_row_t>,int64_t,sq3_err_t>;
+
+enum SQ3_RES_T {
+	SQ3_COLUMNS_DEF,
+	SQ3_ROWS,
+	SQ3_AFFECTED_ROWS,
+	SQ3_ERR
+};
+
+/**
+ * @brief Executes the provided query in the supplied sqlite3 db object.
+ * @param db Already initialized 'sqlite3' handler.
+ * @param query The query to be executed.
+ * @return An 'sq3_result_t' object holding the result, depending on the type of query and result, different
+ *  fields will be populated, in case of success:
+ *   - For DQL stmts COLUMN_DEF and ROWS will hold the columns definitions and the rows from the resultset.
+ *   - For DML stmts the AFFECTED_ROWS will show the number of modified rows.
+ *  In case of failure, ERR field will be populated and others will remain empty.
+ */
+sq3_res_t sqlite3_execute_stmt(sqlite3* db, const std::string& query);
+
+/**
+ * @brief Utility one-liner macro to check for query failure on a 'ext_val_t<T>'.
+ * @param val The 'ext_val_t<T>' to be checked.
+ * @return In case of failure, 'EXIT_FAILURE' after logging the error, continues otherwise.
+ */
+#define CHECK_EXT_VAL(val)\
+	do {\
+		if (val.err) {\
+			diag("%s:%d: Query failed   err=\"%s\"", __func__, __LINE__, val.str.c_str());\
+			return EXIT_FAILURE;\
+		}\
+	} while(0)
 
 /**
  * @brief Holds the result of an `mysql_query_ext_val` operation.
@@ -182,7 +278,7 @@ struct ext_val_t {
 };
 
 /**
- * @brief Specifications of function 'ext_single_row_val' for different types.
+ * @brief Specializations of function 'ext_single_row_val' for different types.
  * @details These functions serve as the extension point for `mysql_query_ext_val`. A new specialization of
  *  the function is required for each type that `mysql_query_ext_val` should support for the default value.
  * @param row The row from which the first value is going to be extracted and parsed.
@@ -231,6 +327,32 @@ ext_val_t<T> mysql_query_ext_val(MYSQL* mysql, const std::string& query, const T
 	 }
 }
 
+template <typename T>
+ext_val_t<T> sq3_query_ext_val(sqlite3* db, const std::string& query, const T& def_val) {
+	const auto& sq3_res { sqlite3_execute_stmt(db, query) };
+	const auto& sq3_err { std::get<SQ3_RES_T::SQ3_ERR>(sq3_res) };
+	const auto& sq3_rows { std::get<SQ3_RES_T::SQ3_ROWS>(sq3_res) };
+
+	if (sq3_err) {
+		return { sq3_err, def_val };
+	} else if (sq3_rows.empty()) {
+		return { -1, def_val };
+	} else {
+		return ext_single_row_val(sq3_rows.front(), def_val);
+	}
+}
+
+template <typename T>
+std::string sq3_get_ext_val_err(const ext_val_t<T>& ext_val) {
+	if (ext_val.err == -1) {
+		return "Received invalid empty resultset/row";
+	} else if (ext_val.err == -2) {
+		return "Failed to parse response value '" + ext_val.str + "'";
+	} else {
+		return std::string { sqlite3_errstr(ext_val.err) };
+	}
+}
+
 /**
  * @brief Extract the error from a `ext_val_t<T>`.
  * @param mysql Already oppened MYSQL connection.
@@ -378,16 +500,19 @@ int create_extra_users(
 	MYSQL* proxysql_admin, MYSQL* mysql_server, const std::vector<user_config>& users_config
 );
 
-std::string tap_curtime();
 /**
  * @brief Returns ProxySQL cpu usage in ms.
- * @param intv The interval in which the CPU usage of ProxySQL is going
- *  to be measured.
- * @param cpu_usage Output parameter with the cpu usage by ProxySQL in
- *  'ms' in the specified interval.
+ * @details The parameter intv, determines the duration of the measuring interval, and by such,
+ *  determines the execution time of the function.
+ * @param cpu_usage Output holding ProxySQL CPU usage ('ms') in the interval.
+ * @param intv The interval in which the CPU usage is going to be measured. ProxySQL internal stats
+ *  tables are used for performing the measurement, this parameter modifies 'admin-stats_system_cpu',
+ *  modifying internal refresh-interval. The special value UINT32_MAX allows to use the current value
+ *  for 'admin-stats_system_cpu', thus not changing current ProxySQL internal measuring interval.
+ *  This is the default value, since testing environments use max resolution anyway.
  * @return 0 if success, -1 in case of error.
  */
-int get_proxysql_cpu_usage(const CommandLine& cl, uint32_t intv, double& cpu_usage);
+int get_proxysql_cpu_usage(const CommandLine& cl, double& cpu_usage, uint32_t intv=UINT32_MAX);
 
 /**
  * @brief Helper struct holding connection options for helper functions creating MySQL connections.
@@ -630,10 +755,28 @@ enum LINE_MATCH_T { POS, LINE, MATCH };
  *  For example, regex '\d+' should become '(\d+)'.
  * @return All the lines found matching the regex.
  */
-std::vector<line_match_t> get_matching_lines(
+std::pair<size_t,std::vector<line_match_t>> get_matching_lines(
 	std::fstream& f_stream, const std::string& regex, bool get_matches=false
 );
 
+/**
+ * @brief Row entries from 'debug_log' table, from debug database.
+ */
+struct debug_entry_t {
+	uint64_t id;
+	time_t time;
+	uint64_t lapse;
+	uint64_t thread;
+	std::string file;
+	uint64_t line;
+	std::string funct;
+	uint64_t modnum;
+	std::string modname;
+	uint64_t verbosity;
+	std::string message;
+	std::string note;
+};
+
 /**
  * @brief Opens a sqlite3 db file located in the supplied path with the provided flags.
  * @param f_path Path to the 'db' file.
@@ -642,30 +785,20 @@ std::vector<line_match_t> get_matching_lines(
  * @return EXIT_SUCCESS in case of success, EXIT_FAILURE otherwise. Error cause is logged.
  */
 int open_sqlite3_db(const std::string& f_path, sqlite3** db, int flags);
-
-using sq3_col_def_t = std::string;
-using sq3_row_t = std::vector<std::string>;
-using sq3_err_t = std::string;
-using sq3_res_t = std::tuple<std::vector<sq3_col_def_t>,std::vector<sq3_row_t>,int64_t,sq3_err_t>;
-
-enum SQ3_RES_T {
-	SQ3_COLUMNS_DEF,
-	SQ3_ROWS,
-	SQ3_AFFECTED_ROWS,
-	SQ3_ERR
-};
-
 /**
- * @brief Executes the provided query in the supplied sqlite3 db object.
- * @param db Already initialized 'sqlite3' handler.
- * @param query The query to be executed.
- * @return An 'sq3_result_t' object holding the result, depending on the type of query and result, different
- *  fields will be populated, in case of success:
- *   - For DQL stmts COLUMN_DEF and ROWS will hold the columns definitions and the rows from the resultset.
- *   - For DML stmts the AFFECTED_ROWS will show the number of modified rows.
- *  In case of failure, ERR field will be populated and others will remain empty.
+ * @brief Builds a debug_entry_t from an sq3_row_t from 'debug_log' table.
+ * @param row The row to map to a 'debug_entry_t' struct.
+ * @return A pair of kind `{err_code, debug_entry_t}`.
  */
-sq3_res_t sqlite3_execute_stmt(sqlite3* db, const std::string& query);
+rc_t<debug_entry_t> build_debug_entry(const sq3_row_t& row);
+/**
+ * @brief Retrieves a list of debug entries matching the supplied conditions.
+ * @param db An already opened connection to a 'SQLite3' database.
+ * @param conds Conditions for the WHERE clause for row filtering, no filtering if empty. E.g: 'id >
+ *  $timestamp AND file="foo.cpp"'.
+ * @return A pair of kind `{err_code, matched_rows}`.
+ */
+rc_t<std::vector<debug_entry_t>> sq3_get_debug_entries(sqlite3* db, const std::string& conds);
 
 /**
  * @brief If found returns the element index, -1 otherwise.
@@ -725,6 +858,31 @@ struct POOL_STATS_IDX {
 	};
 };
 
+struct hg_pool_st_t {
+	uint32_t hostgroup;
+	uint32_t conn_used;
+	uint32_t conn_free;
+	uint32_t conn_ok;
+	uint32_t conn_err;
+	uint32_t max_conn_used;
+	uint32_t queries;
+};
+
+/**
+ * @brief A more complex type specialization of 'ext_single_row_val'.
+ * @details For internal use of function 'get_conn_pool_hg_stats'.
+ * @param row The row from which the first value is going to be extracted and parsed.
+ * @param def_val The default value to use in case of failure to extract.
+ * @return An `ext_val_t<T>` where T is the type of the provided default value.
+ */
+ext_val_t<hg_pool_st_t> ext_single_row_val(const mysql_res_row& row, const hg_pool_st_t& def_val);
+/**
+ * @brief Fetches the stats from a particular hostgroup.
+ * @param admin An already opened connection to MySQL admin.
+ * @return An `ext_val_t` wrapping the extracted values.
+ */
+ext_val_t<hg_pool_st_t> get_conn_pool_hg_stats(MYSQL* admin, uint32_t hg);
+
 /**
  * @brief Dumps a resultset with fields from the supplied hgs from 'stats_mysql_connection_pool'.
  * @details The fetched fields are 'hostgroup,ConnUsed,ConnFree,ConnOk,ConnERR,MaxConnUsed,Queries'.
@@ -741,6 +899,37 @@ using pool_state_t = std::map<uint32_t,mysql_row_t>;
  * @return A pair of the shape {err_code, pool_state_t}.
  */
 std::pair<int,pool_state_t> fetch_conn_stats(MYSQL* admin, const std::vector<uint32_t> hgs);
+
+/**
+ * @brief Waits for a generic condition.
+ * @details Wait finishes by a non-zero return code by the condition or by timeout.
+ * @param cond Condition to be evaluated at each wait interval.
+ * @param to_us Timeout at which to stop the wait, in microseconds.
+ * @param delay_us Delay between wait intervals, in microseconds.
+ * @return 0 for success, non-zero return code from 'cond' for failure, or INT_MIN for timeout.
+ */
+template <typename T>
+rc_t<T> wait_for_cond(const std::function<rc_t<T>()>& cond, uint32_t to_us, uint32_t delay_us) {
+	int rc { 0 };
+	T res {};
+	uint32_t waited { 0 };
+
+	while (waited < to_us) {
+		std::tie(rc, res) = cond();
+
+		if (rc == 0) {
+			usleep(delay_us);
+			waited += delay_us;
+		} else if (rc < 0) {
+			break;
+		} else {
+			return { 0, res };
+		}
+	}
+
+	return { INT_MIN, res };
+}
+
 /**
  * @brief Waits until the condition specified by the 'query' holds, or 'timeout' is reached.
  * @details Several details about the function impl:
@@ -754,8 +943,6 @@ std::pair<int,pool_state_t> fetch_conn_stats(MYSQL* admin, const std::vector<uin
  */
 int wait_for_cond(MYSQL* mysql, const std::string& query, uint32_t timeout);
 
-using check_res_t = std::pair<int,std::string>;
-
 /**
  * @brief Waits for multiple conditions to take place before returning.
  * @param mysql Already oppened connection in which to execute the queries.
@@ -763,14 +950,14 @@ using check_res_t = std::pair<int,std::string>;
  * @param to Timeout in which all the conditions should be accomplished.
  * @return Vector of pairs of shape '{err, check}'.
  */
-std::vector<check_res_t> wait_for_conds(MYSQL* mysql, const std::vector<std::string>& qs, uint32_t to);
+std::vector<rc_t<std::string>> wait_for_conds(MYSQL* mysql, const std::vector<std::string>& qs, uint32_t to);
 
 /**
  * @brief Reduces a vector of 'check_res_t' to either success or failure.
  * @param chks Vector to be fold into single value.
  * @return -1 in case a check failed to execute, 1 if any check timedout, 0 for success.
  */
-int proc_wait_checks(const std::vector<check_res_t>& chks);
+int proc_wait_checks(const std::vector<std::pair<int,std::string>>& chks);
 
 /**
  * @brief Encapsulates a server address.
