@@ -20,7 +20,7 @@
 #include <cstring>
 #include <unistd.h>
 
-#include <mysql.h>
+#include "mysql.h"
 
 #include "tap.h"
 #include "command_line.h"
