@@ -7,8 +7,8 @@
 #include <unistd.h>
 #include <string>
 #include <thread>
-#include <mysql.h>
-#include <mysql/mysqld_error.h>
+#include "mysql.h"
+#include "mysqld_error.h"
 #include "tap.h"
 #include "command_line.h"
 #include "utils.h" 
@@ -99,7 +99,7 @@ int main(int argc, char** argv) {
 
 	unsigned long fetched_row_count = 0;
 
-	while (row = mysql_fetch_row(mysql_result)) {
+	while ((row = mysql_fetch_row(mysql_result))) {
 		fetched_row_count++;
 		usleep(10);
 	}
