@@ -17,7 +17,7 @@
 #include <unistd.h>
 #include <vector>
 
-#include <mysql.h>
+#include "mysql.h"
 
 #include "tap.h"
 #include "command_line.h"
