@@ -1,8 +1,8 @@
-
 #include <unistd.h>
-#include <atomic>
 #include <vector>
 #include <string>
+#include <utility>
+#include <vector>
 
 #include "mysql.h"
 #include "tap.h"
@@ -15,6 +15,9 @@
 //#define BACKEND_SERVER_USER	"root"
 //#define BACKEND_SERVER_PASS "root"
 
+using std::vector;
+using std::pair;
+
 #define MYSQL_QUERY__(mysql, query) \
 	do { \
 		if (mysql_query(mysql, query)) { \
@@ -49,87 +52,6 @@ __exit:
 	return mysql;
 }
 
-/**
- * @brief Helper function to verify that the sync of a table (or variable) have been performed.
- *
- * @param r_proxy_admin An already opened connection to ProxySQL.
- * @param queries Queries to be executed that should return a **non-zero** value after the sync has taken place.
- * @param sync_timeout Timeout for the sync to happen.
- *
- * @return EXIT_SUCCESS in case of success, otherwise:
- *  - '-1' if a query against Admin fails to be performed (failure is logged).
- *  - '-2' if timeout expired without sync being completed.
- */
-int sync_checker(MYSQL* r_proxy_admin, const std::vector<std::string>& queries, uint32_t sync_timeout) {
-	bool not_synced_query = false;
-	uint waited = 0;
-
-	while (waited < sync_timeout) {
-		not_synced_query = false;
-
-		// Check that all the entries have been synced
-		for (const auto& query : queries) {
-			int q_res = mysql_query(r_proxy_admin, query.c_str());
-			if (q_res != EXIT_SUCCESS) {
-				fprintf(stderr, "File %s, line %d, Error: %s\n", __FILE__, __LINE__, mysql_error(r_proxy_admin));
-				return -1;
-			}
-
-			MYSQL_RES* proxysql_servers_res = mysql_store_result(r_proxy_admin);
-			MYSQL_ROW row = mysql_fetch_row(proxysql_servers_res);
-			int row_value = atoi(row[0]);
-			mysql_free_result(proxysql_servers_res);
-
-			if (row_value == 0) {
-				not_synced_query = true;
-				break;
-			}
-		}
-
-		if (not_synced_query) {
-			waited += 1;
-			sleep(1);
-		} else {
-			break;
-		}
-	}
-
-	if (not_synced_query) {
-		return -2;
-	} else {
-		return EXIT_SUCCESS;
-	}
-}
-
-int check_nodes_sync(
-	const CommandLine& cl, const std::vector<mysql_res_row>& core_nodes, const std::string& check_query, uint32_t sync_timeout
-) {
-	int ret_status = EXIT_FAILURE;
-
-	for (const auto& node : core_nodes) {
-		const std::string host { node[0] };
-		const int port = std::stol(node[1]);
-
-		MYSQL* c_node_admin = mysql_init(NULL);
-		if (!mysql_real_connect(c_node_admin, host.c_str(), cl.admin_username, cl.admin_password, NULL, port, NULL, 0)) {
-			fprintf(stderr, "File %s, line %d, Error: %s\n", __FILE__, __LINE__, mysql_error(c_node_admin));
-			goto __exit;
-		}
-
-		int not_synced = sync_checker(c_node_admin, { check_query }, sync_timeout);
-		if (not_synced != EXIT_SUCCESS) {
-			const std::string err_msg { "Node '"  + host + ":" + std::to_string(port) + "' sync timed out" };
-			fprintf(stderr, "File %s, line %d, Error: `%s`\n", __FILE__, __LINE__, err_msg.c_str());
-			goto __exit;
-		}
-	}
-
-	ret_status = EXIT_SUCCESS;
-
-__exit:
-	return ret_status;
-}
-
 int insert_mysql_servers_records(MYSQL* proxy_admin, const std::vector<mysql_server_tuple>& insert_mysql_servers_values, 
 	const std::vector<replication_hostgroups_tuple>& insert_replication_hostgroups_values) {
 
@@ -535,7 +457,7 @@ cleanup:
 
 int test_read_only_offline_hard_servers(MYSQL* proxy_admin, const CommandLine& cl, bool isolate_primary_node) {
 
-	std::vector<mysql_res_row> core_nodes;
+	pair<int,vector<srv_addr_t>> core_nodes;
 	std::string check_no_primary_query;
 
 	if (isolate_primary_node) {
@@ -553,10 +475,9 @@ int test_read_only_offline_hard_servers(MYSQL* proxy_admin, const CommandLine& c
 		// 2. Remove primary from Core nodes
 		MYSQL_QUERY__(proxy_admin, "DELETE FROM proxysql_servers WHERE hostname=='127.0.0.1' AND PORT==6032");
 		MYSQL_QUERY__(proxy_admin, "LOAD PROXYSQL SERVERS TO RUNTIME");
-		MYSQL_QUERY__(proxy_admin, "SELECT hostname,port FROM proxysql_servers");
-		MYSQL_RES* my_res = mysql_store_result(proxy_admin);
-		core_nodes = { extract_mysql_rows(my_res) };
-		mysql_free_result(my_res);
+
+		core_nodes = fetch_cluster_nodes(proxy_admin);
+		if (core_nodes.first) { goto cleanup; }
 
 		// 3. Wait for all Core nodes to sync (confirm primary out of core nodes)
 		string_format(
@@ -564,7 +485,7 @@ int test_read_only_offline_hard_servers(MYSQL* proxy_admin, const CommandLine& c
 			check_no_primary_query, cl.host, cl.admin_port
 		);
 
-		int check_res = check_nodes_sync(cl, core_nodes, check_no_primary_query, SYNC_TIMEOUT);
+		int check_res = check_nodes_sync(cl, core_nodes.second, check_no_primary_query, SYNC_TIMEOUT);
 		if (check_res != EXIT_SUCCESS) {
 			goto cleanup;
 		}
@@ -601,9 +522,9 @@ cleanup:
 				insert_query, cl.host, cl.admin_port
 			);
 
-			for (const auto& row : core_nodes) {
-				const std::string host{ row[0] };
-				const int port = std::stol(row[1]);
+			for (const auto& row : core_nodes.second) {
+				const std::string host{ row.host };
+				const int port = row.port;
 				MYSQL* c_node_admin = mysql_init(NULL);
 
 				diag("RESTORING: Inserting into node '%s:%d'", host.c_str(), port);
@@ -637,7 +558,7 @@ cleanup:
 			);
 
 			// Wait for the other nodes to sync ProxySQL servers to include Primary
-			int check_res = check_nodes_sync(cl, core_nodes, check_no_primary_query, SYNC_TIMEOUT);
+			int check_res = check_nodes_sync(cl, core_nodes.second, check_no_primary_query, SYNC_TIMEOUT);
 			if (check_res != EXIT_SUCCESS) { return EXIT_FAILURE; }
 
 			// Recover the old ProxySQL servers from backup in primary
