@@ -13,7 +13,7 @@
 
 #include "json.hpp"
 #include "mysql.h"
-#include "mysql/mysqld_error.h"
+#include "mysqld_error.h"
 #include "tap.h"
 #include "command_line.h"
 #include "utils.h" 
